Package: PhIPData
Type: Package
Title: Container for PhIP-Seq Experiments
Version: 1.21.0
Authors@R: c(person("Athena", "Chen", email = "achen70@jhu.edu", 
    role = c("aut", "cre"), comment = c(ORCID = "0000-0001-6900-2264")),
    person("Rob", "Scharpf", email = "rscharpf@jhu.edu", 
    role = c("aut")), 
    person("Ingo", "Ruczinski", email = "ingo@jhu.edu", 
    role = c("aut")))
Description: PhIPData defines an S4 class for phage-immunoprecipitation 
    sequencing (PhIP-seq) experiments. Buliding upon the 
    RangedSummarizedExperiment class, PhIPData enables users to coordinate 
    metadata with experimental data in analyses. Additionally, PhIPData 
    provides specialized methods to subset and identify beads-only samples, 
    subset objects using virus aliases, and use existing peptide libraries to 
    populate object parameters. 
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.1.0), SummarizedExperiment (>= 1.3.81)
Imports: BiocFileCache, BiocGenerics, methods, GenomicRanges, IRanges,
        S4Vectors, edgeR, cli, utils
Suggests: BiocStyle, testthat, knitr, rmarkdown, covr, dplyr, readr,
        withr
biocViews: Infrastructure, DataRepresentation, Sequencing, Coverage
BugReports: https://github.com/athchen/PhIPData/issues
Collate: 'defineBeads.R' 'PhIPData-class.R' 'alias.R' 'library.R'
        'subset.R' 'summaries.R' 'zzz.R'
RoxygenNote: 7.1.2
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/PhIPData
git_branch: devel
git_last_commit: 6c7acec
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:20:19 UTC; biocbuild
Author: Athena Chen [aut, cre] (ORCID: <https://orcid.org/0000-0001-6900-2264>),
  Rob Scharpf [aut],
  Ingo Ruczinski [aut]
Maintainer: Athena Chen <achen70@jhu.edu>

# Generated by roxygen2: do not edit by hand

export("logfc<-")
export("peptideInfo<-")
export("prob<-")
export("sampleInfo<-")
export(PhIPData)
export(deleteAlias)
export(getAlias)
export(getBeadsName)
export(getLibrary)
export(librarySize)
export(listLibrary)
export(logfc)
export(makeLibrary)
export(peptideInfo)
export(prob)
export(propReads)
export(removeLibrary)
export(sampleInfo)
export(setAlias)
export(setBeadsName)
export(subsetBeads)
exportMethods("assay<-")
exportMethods("assays<-")
exportMethods("counts<-")
exportMethods("logfc<-")
exportMethods("peptideInfo<-")
exportMethods("prob<-")
exportMethods("sampleInfo<-")
exportMethods(counts)
exportMethods(logfc)
exportMethods(peptideInfo)
exportMethods(prob)
exportMethods(sampleInfo)
import(S4Vectors)
import(SummarizedExperiment)
import(methods)
importClassesFrom(edgeR,DGEList)
importFrom(BiocGenerics,"counts<-")
importFrom(BiocGenerics,counts)
importFrom(GenomicRanges,GRanges)
importFrom(IRanges,IRanges)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,isEmpty)
importFrom(S4Vectors,setValidity2)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_alert_warning)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# PhIPData

<!-- badges: start -->

[![R-CMD-check](https://github.com/athchen/PhIPData/workflows/R-CMD-check/badge.svg)](https://github.com/athchen/PhIPData/actions)
[![BioC
status](http://www.bioconductor.org/shields/build/release/bioc/PhIPData.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/PhIPData)
[![Codecov test
coverage](https://codecov.io/gh/athchen/PhIPData/branch/main/graph/badge.svg)](https://codecov.io/gh/athchen/PhIPData?branch=main)
<!-- badges: end -->

The `PhIPData` class is used to store experimental results from
phage-immunoprecipitation sequencing (PhIP-set) experiments in a
matrix-like container.

Building on the
[`RangedSummarizedExperiment`](https://bioconductor.org/packages/release/bioc/html/SummarizedExperiment.html)
class, `PhIPData` contains all of functionality of
`SummarizedExperiments` and includes additional operations to facilitate
analysis with PhIP-seq data. Like `SummarizedExperiments`, a key feature
of `PhIPData` is the coordination of metadata when subsetting `PhIPData`
objects. For example, if you wanted to examine experimental data for
peptides from one particular virus, you can subset the experimental data
and the associated peptide annotation with one command. This ensures all
metadata (for samples, peptides, etc.) remain synced with the
experimental data throughout analysis.

## Installation

We recommend installing the stable release version of `PhIPData` in
Bioconductor. This can be done using `BiocManager`:

``` r
if (!require("BiocManager"))
    install.packages("BiocManager")
    
BiocManager::install("PhIPData")
```

To load the package:

``` r
library(PhIPData)
```

## Components of a PhIPData Object

As reflected in the figure below, the structure of a `PhIPData` object
is nearly identical to the structure of a
`SummarizedExperiment`/`RangedSummarizedExperiment` object.

Each object contains at least three assays of data. These assays are:

-   `counts`: matrix of raw read counts,
-   `logfc`: matrix of log10 estimated fold-changes (in comparison to
    negative control samples),
-   `prob`: matrix of probabilities (p-values or posterior
    probabilities) associated with whether a sample shows an enriched
    antibody response to the particular peptide.

The rows of a `PhIPData` object represent peptides of interest and the
columns represent samples. Sample and peptide metadata are stored in
`DataFrame`s. Each row of the metadata `DataFrame` specifies the
peptide/sample, and the columns represent different features associated
with the peptides/samples.

In addition to sample- and peptide-specific metadata, experimental
metadata such as associated papers, experimental parameters, sequencing
dates, etc. are stored in a list-like component named `metadata`.

<div class="figure">

<img src="vignettes/extras/PhIPData.png" alt="Schematic of a PhIPData object. Commands used to access each component of the object are listed underneath its visual representation. Code in black indicates functions specific to `PhIPData` objects while functions in red extend `SummarizedExperiment` functions. Here, `pd` is a generic `PhIPData` object." width="\maxwidth" />
<p class="caption">
Schematic of a PhIPData object. Commands used to access each component
of the object are listed underneath its visual representation. Code in
black indicates functions specific to `PhIPData` objects while functions
in red extend `SummarizedExperiment` functions. Here, `pd` is a generic
`PhIPData` object.
</p>

</div>

## Example Use

For an example of how to use `PhIPData` objects, please see the package
vignette.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  comment = "##"
)

## ----eval = FALSE-------------------------------------------------------------
# if (!require("BiocManager"))
#     install.packages("BiocManager")
# 
# BiocManager::install("PhIPData")

## ----include = TRUE, results = "hide", message = FALSE, warning = FALSE-------
library(PhIPData)

## ----echo = FALSE, fig.cap = "Schematic of a PhIPData object. Commands used to access each component of the object are listed underneath its visual representation. Code in black indicates functions specific to `PhIPData` objects while functions in red extend `SummarizedExperiment` functions. Here, `pd` is a generic `PhIPData` object.", out.width = "\\maxwidth"----
knitr::include_graphics("extras/PhIPData.png")

## ----setup, warning = FALSE, message = FALSE----------------------------------
library(dplyr)
library(readr)

## -----------------------------------------------------------------------------
set.seed(20210120)

# Read in peptide metadata -------------
virscan_file <- system.file("extdata", "virscan.tsv", package = "PhIPData")
virscan_info <- readr::read_tsv(virscan_file,
                                col_types = readr::cols(
                                  pep_id = readr::col_character(),
                                  pro_id = readr::col_character(),
                                  pos_start = readr::col_double(),
                                  pos_end = readr::col_double(),
                                  UniProt_acc = readr::col_character(),
                                  pep_dna = readr::col_character(),
                                  pep_aa = readr::col_character(),
                                  pro_len = readr::col_double(),
                                  taxon_id = readr::col_double(),
                                  species = readr::col_character(),
                                  genus = readr::col_character(),
                                  product = readr::col_character()
                                )) %>%
  as.data.frame()

# Simulate data -------------
n_samples <- 5L
n_peps <- nrow(virscan_info)

counts_dat <- matrix(sample(1:1e6, n_samples*n_peps, replace = TRUE),
                 nrow = n_peps)
logfc_dat <- matrix(rnorm(n_samples*n_peps, mean = 0, sd = 10), 
                    nrow = n_peps)
prob_dat <- matrix(rbeta(n_samples*n_peps, shape1 = 1, shape2 = 1), 
                   nrow = n_peps)

# Sample metadata -------------
sample_meta <- data.frame(sample_name = paste0("sample", 1:n_samples),
                          gender = sample(c("M", "F"), n_samples, TRUE),
                          group = sample(c("ctrl", "trt", "beads"), n_samples, TRUE))

# Set row/column names -------------
rownames(counts_dat) <- rownames(logfc_dat) <-
  rownames(prob_dat) <- rownames(virscan_info) <-
  paste0("pep_", 1:n_peps)

colnames(counts_dat) <- colnames(logfc_dat) <-
  colnames(prob_dat) <- rownames(sample_meta) <-
  paste0("sample_", 1:n_samples)

# Experimental metadata -------------
exp_meta <- list(date_run = as.Date("2021/01/20"), 
                 reads_per_sample = colSums(counts_dat))

## -----------------------------------------------------------------------------
phip_obj <- PhIPData(counts_dat, logfc_dat, prob_dat, 
                     virscan_info, sample_meta, 
                     exp_meta)

phip_obj

## -----------------------------------------------------------------------------
assays(phip_obj)

head(assays(phip_obj)$counts)   # Returns the same as assays(phip_obj)[["counts"]]

## -----------------------------------------------------------------------------
head(assay(phip_obj, "logfc"))   # Returns the same as assay(phip_obj, 2)

## -----------------------------------------------------------------------------
head(counts(phip_obj))
head(logfc(phip_obj))
head(prob(phip_obj))

## ----error = T----------------------------------------------------------------
replacement_dat <- matrix(1, nrow = n_peps, ncol = n_samples)

# Replace the counts matrix -------------
head(counts(phip_obj))
counts(phip_obj) <- replacement_dat
head(counts(phip_obj))

# Add a new assay -----------
head(assay(phip_obj, "new_assay"))
assay(phip_obj, "new_assay") <- replacement_dat
head(assay(phip_obj, "new_assay"))

# Returns and error because `counts`, `logfc`, and `prob` must be in the 
# assays of a PhIPData object
assays(phip_obj) <- list(new_assay1 = replacement_dat, 
                         new_assay2 = replacement_dat)

## -----------------------------------------------------------------------------
# Only showing 2 columns for easier viewing
peptideInfo(phip_obj)[, 8:9]

## -----------------------------------------------------------------------------
sampleInfo(phip_obj)

## -----------------------------------------------------------------------------
phip_obj$group

## -----------------------------------------------------------------------------
metadata(phip_obj)

## -----------------------------------------------------------------------------
phip_obj[1:10, 1:2]

## -----------------------------------------------------------------------------
phip_obj[, phip_obj$group == "beads"]

## -----------------------------------------------------------------------------
ebv_sub <- subset(phip_obj, grepl("Epstein-Barr virus", species))

peptideInfo(ebv_sub)[, "species"]

## -----------------------------------------------------------------------------
subsetBeads(phip_obj)

## -----------------------------------------------------------------------------
librarySize(phip_obj)

## -----------------------------------------------------------------------------
head(propReads(phip_obj))

## -----------------------------------------------------------------------------
# Save virscan_info as the human_virus library
makeLibrary(virscan_info, "human_virus")

## -----------------------------------------------------------------------------
PhIPData(counts = counts_dat, 
         sampleInfo = sample_meta,
         peptideInfo = getLibrary("human_virus"))

## -----------------------------------------------------------------------------
removeLibrary("human_virus")

## ----error = T----------------------------------------------------------------
# Create alias for HIV ----------
setAlias("hiv", "[Hh]uman immunodeficiency virus")

# Use alias ----------
hiv_sub <- subset(phip_obj, grepl(getAlias("hiv"), species))
peptideInfo(hiv_sub)[, "species"]

# Remove alias from database -----------
deleteAlias("hiv")

# The following command returns an error that the virus does
# not exist in the alias database. 
subset(phip_obj, grepl(getAlias("hiv"), species))

## -----------------------------------------------------------------------------
# PhIPData to DGEList
as(phip_obj, "DGEList")
as(phip_obj, "DataFrame")

## -----------------------------------------------------------------------------
sessionInfo()

pkg_env <- new.env()

.createDefaults <- function() {

    # Create default variable for beads name
    beads_path <- BiocFileCache::bfcnew(
        pkg_env$beer_cache, "beads_name",
        ext = ".rds"
    )
    saveRDS("beads", beads_path)

    # Copy default alias list to cache
    BiocFileCache::bfcadd(pkg_env$beer_cache, "alias",
        system.file(package = "PhIPData", "extdata/alias.rda"),
        action = "copy"
    )

    # Copy small VirScan library to cache
    BiocFileCache::bfcadd(pkg_env$beer_cache, "library_virscan",
        system.file(package = "PhIPData", "libraries", "virscan.rds"),
        action = "copy"
    )
    TRUE
}

.onLoad <- function(libname, pkgname) {

    ## Load cache
    cache_path <- tools::R_user_dir("beer", which = "cache")
    assign("beer_cache",
        BiocFileCache::BiocFileCache(cache_path, ask = FALSE),
        envir = pkg_env
    )

    ## If cache does not exist, add defaults
    if (length(pkg_env$beer_cache) == 0) {
        tmp <- .createDefaults()
    }

    ## Load beads name
    beads_path <- BiocFileCache::bfcquery(pkg_env$beer_cache, "beads_name")$rpath
    pkg_env$BEADS_NAME <- readRDS(beads_path)

    ## Load alias
    alias_path <- BiocFileCache::bfcquery(pkg_env$beer_cache, "alias")$rpath
    load(alias_path, envir = pkg_env)
}

#' Script to generate `alias.rda` in `inst/extdata`
#'
#' The `alias.rda` file contains a single dataframe with alias-pattern pairs.
#' Below is code to generate the initial alias database. For more information on
#' working with the alias database, see \code{?aliases}.
alias <- data.frame(alias = c("EBV", "HIV", "HPV"),
                    pattern = c("Epstein-Barr", "Human immunodeficiency virus",
                                "Human papillomavirus"))

save(list = "alias", file = "inst/extdata/alias.rda")

#' Peptide libraries
#'
#' @description PhIP-Seq experiments often use identical peptide libraries
#' different cohorts. These functions enable the user to conveniently reuse
#' tidied libraries.
#'
#' @details Each library is stored as a \linkS4class{DataFrame} in .rds file.
#' New libraries can be stored for future use with the \code{makeLibrary}
#' function.
#'
#' @param name name of the library
#' @param library a \code{matrix}, \code{data.frame}, or
#'     \linkS4class{DataFrame} with the peptide information for the
#'     specified library.
#'
#' @return \code{getLibrary} returns a \linkS4class{DataFrame} corresponding to
#' the peptide information for the specified library.
#'
#' @examples
#' ## Create a new library
#' pep_meta <- data.frame(species = c(
#'     rep("human immunodeficiency virus", 3),
#'     rep("Epstein-Barr virus", 2)
#' ))
#' makeLibrary(pep_meta, "new_library")
#'
#' ## Use new library
#' counts_dat <- matrix(1:10, nrow = 5)
#' phip_obj <- PhIPData(
#'     counts = counts_dat,
#'     peptideInfo = getLibrary("new_library")
#' )
#'
#' ## List libraries
#' listLibrary()
#'
#' ## Delete created library
#' removeLibrary("new_library")
#' @name peptideLibraries
#' @include PhIPData-class.R
NULL

#' @describeIn peptideLibraries return a \linkS4class{DataFrame} with the
#' peptide information corresponding to the library.
#'
#' @export
getLibrary <- function(name) {
    path <- BiocFileCache::bfcquery(
        pkg_env$beer_cache,
        paste0("library_", name)
    )$rpath
    readRDS(path)
}

#' @describeIn peptideLibraries create and store a \linkS4class{DataFrame} with
#' the specified peptide information.
#'
#' @export
makeLibrary <- function(library, name) {
    ## Check if library exists
    query_out <- BiocFileCache::bfcquery(
        pkg_env$beer_cache,
        paste0("library_", name)
    )$rpath

    ## If the library exists, confirm that the file should be overwritten
    write <- if (length(query_out) != 0) {
        utils::menu(c("Yes", "No"),
            title = paste0(
                "The library files already exists. ",
                "Do you want to overwrite the file?"
            )
        )
    } else {
        1
    }

    if (write == 1) {
        # Define library path
        library_path <- ifelse(
            length(query_out) == 0,
            BiocFileCache::bfcnew(
                pkg_env$beer_cache,
                paste0("library_", name),
                ext = ".rds"
            ),
            query_out
        )
        saveRDS(library, file = library_path)
    }
}

#' @describeIn peptideLibraries delete stored libraries
#'
#' @export
removeLibrary <- function(name) {

    ## Check if library exists
    query_out <- BiocFileCache::bfcquery(
        pkg_env$beer_cache,
        paste0("library_", name)
    )$rid

    if (length(query_out) == 0) {
        cli::cli_alert_warning("Library not found. No libraries removed.")
    } else {
        BiocFileCache::bfcremove(pkg_env$beer_cache, query_out)
    }
}


#' @describeIn peptideLibraries list all available libraries
#'
#' @export
listLibrary <- function() {

    ## Check if library exists
    BiocFileCache::bfcquery(
        pkg_env$beer_cache,
        "library_"
    )$rname
}

#' Defining how beads-only samples are encoded.
#'
#' @description \code{getBeadsName} and \code{setBeadsName} are two function to
#' get and set the string that encodes which samples are beads-only samples.
#' Information about beads-only samples are stored in the \code{groups} column
#' of \code{sampleInfo}.
#'
#' @details If \code{name} is of length greater than one, only the first element
#' of the vector is used. Non-character values of \code{name} are first coerced
#' into strings.
#'
#' @examples
#' ## Returns the default string, "beads"
#' getBeadsName()
#'
#' ## Not run since it changes defaults/user settings
#' \dontrun{
#' setBeadsName("beads-only")
#' }
#'
#' @name defineBeads
NULL

#' @describeIn defineBeads function that returns a string corresponding to how
#' beads-only samples are encoded.
#'
#' @return a string indicating how beads-only samples are encoded.
#' @export
getBeadsName <- function() {
    get("BEADS_NAME", envir = pkg_env)
}

#' @describeIn defineBeads function to set the string that indicates which
#' samples are beads-only samples in the \code{groups} column of
#' \code{sampleInfo}.
#'
#' @param name a string indicating how beads-only samples are encoded.
#' @export
setBeadsName <- function(name) {
    if (length(name) > 1) {
        cli::cli_alert_warning(paste0(
            "Input has length larger than one. ",
            "Using only the first element."
        ))
    }
    if (typeof(name) != "character") {
        cli::cli_alert_warning(paste0(
            "Input is of type ", typeof(name), ". ",
            "Coercing to character."
        ))
    }

    name <- as.character(name[1])

    if (is.na(name)) {
        stop("Beads cannot be specified via NA.")
    } else {
        assign("BEADS_NAME", name, envir = pkg_env)
        beads_path <- BiocFileCache::bfcquery(
            pkg_env$beer_cache, "beads_name"
        )$rpath
        saveRDS(name, beads_path)
    }
}

context("Varying strings can be used to indicate beads-only samples")

test_that("varying strings can be used to indicate beads-only samples", {
    expect_equal(getBeadsName(), "beads")

    # Test setters
    setBeadsName("BEADS_ONLY")
    expect_equal(getBeadsName(), "BEADS_ONLY")
    setBeadsName(1)
    expect_equal(getBeadsName(), "1")
    expect_error(setBeadsName(NA), "Beads cannot be specified via NA.")

    # Clean environment
    setBeadsName("beads")
})

#' VirScan Library
#'
#' Below is the code to create a miniature peptide library to demontrate some
#' functionality of the PhIPData class. The full VirScan library, as described
#' in Xu et. al, consists of peptides that tile proteins from all human viruses.
#' The full library annotations can be found in the \code{pep_anno} object of the
#' \code{data_raw/hiv_data.rda} found
#' \href{https://github.com/athchen/ktsp_paper/}{here}.
#'
#' @seealso https://github.com/athchen/ktsp_paper/
#' @seealso Xu, G. J., Kula, T., Xu, Q., Li, M. Z., Vernon, S. D., Ndung'u, T.,
#'      Ruxrungtham, K., Sanchez, J., Brander, C., Chung, R. T.,
#'      O'Connor, K. C., Walker, B., Larman, H. B., & Elledge, S. J. (2015).
#'      Viral immunology. Comprehensive serological profiling of human
#'      populations using a synthetic human virome. Science
#'      (New York, N.Y.), 348(6239), aaa0698.
#'      https://doi.org/10.1126/science.aaa0698

## Download data from the link below.
## "https://github.com/athchen/ktsp_paper/raw/master/data_raw/hiv_data.rda"

library(readr)
library(dplyr)

load("hiv_data.rda")

set.seed(20210106)
pep_anno %>%
    group_by(species) %>%
    sample_n(ceiling(n()/100)) %>%
    write_tsv(file = "inst/extdata/virscan.tsv")

utils::globalVariables(c("alias"))

#' Using aliases to subset virus data
#'
#' @description Rather than typing out full viruses names or repeating
#' regexpressions, users can use aliases as a convenient tool to subset
#' \code{PhIPData} objects by viral species.
#'
#' @details Aliases are cached to an rda file containing only a
#' \code{data.frame} with two columns: \code{alias} and \code{pattern}. The
#' \code{alias} column contains the alias while the \code{pattern} column
#' contains the corresponding regexpression of interest.
#'
#' Once an alias is added to the database, it can always be accessed once the
#' package is loaded. It is recommended to use the functions \code{setAlias}
#' and \code{deleteAlias}. If an alias already exists in the database,
#' \code{setAlias} replaces the matched pattern. If an alias does not exist
#' in the database, \code{getAlias} returns \code{NA_character_}.
#'
#' @param virus character vector of the alias
#' @param pattern character vector of regexpressions corresponding to the alias
#'
#' @return \code{getAlias()} returns a vector of regexpressions corresponding to
#' queried inputs. The returned vector is the same length as the input vector.
#' Queries that do not exist in the database return \code{NA_character_}.
#'
#' @examples
#'
#' ## Edit and modify aliases in the database
#' setAlias("test_virus", "test_pattern")
#' getAlias("test_virus")
#' setAlias("test_virus", "test_pattern2")
#' getAlias("test_virus")
#' deleteAlias("test_virus")
#'
#' ## Edit and modify multiple aliases at once.
#' setAlias(c("virus_1", "virus_2"), c("pattern_1", "pattern_2"))
#' getAlias(c("virus_1", "virus_2"))
#' deleteAlias(c("virus_1", "virus_2"))
#'
#' ## Example of how to subset HIV using `getAlias`
#' ## Often, it is useful to set the `ignore.case` of `grep`/`grepl` to TRUE.
#' counts_dat <- matrix(1:10, nrow = 5)
#' peptide_meta <- data.frame(species = c(
#'     rep("Epstein-Barr virus", 2),
#'     rep("human immunodeficiency virus", 3)
#' ))
#'
#' phip_obj <- PhIPData(counts = counts_dat, peptideInfo = peptide_meta)
#' subset(phip_obj, grepl(getAlias("HIV"), species, ignore.case = TRUE))
#' @name aliases
#'
#' @include PhIPData-class.R
NULL

.getOneAlias <- function(virus) {
    if (!virus %in% get("alias", envir = pkg_env)$alias) {
        NA_character_
    } else {
        get("alias", pkg_env)$pattern[get("alias", pkg_env)$alias == virus]
    }
}

#' @describeIn aliases return a regexpression corresponding to the alias.
#' @export
getAlias <- function(virus) {
    vapply(virus, .getOneAlias, character(1), USE.NAMES = FALSE)
}


#' @describeIn aliases define/modify the regexpression for an alias.
#' @export
setAlias <- function(virus, pattern) {
    if (length(virus) != length(pattern)) {
        stop("Input vector lengths are unequal.")
    }

    ## Create temporary copy for convenience
    current_alias <- get("alias", envir = pkg_env)

    ## Look at whether any viruses need to be added or changed
    ##    new_viruses: viruses to be added
    ##    exist_viruses: viruses that exist in the database
    ##        (may have the same patterns)
    ##    replace_viruses: subset of exist viruses that need to have the pattern
    ##        changed
    new_viruses <- virus[!virus %in% current_alias$alias]
    exist_viruses <- setdiff(virus, new_viruses)
    current_pattern <- vapply(exist_viruses, function(x) {
        pattern <- current_alias$pattern[current_alias$alias == x]
        if (length(pattern) == 0) NA else pattern
    }, character(1))
    replace_viruses <- exist_viruses[current_pattern !=
        pattern[virus == exist_viruses]]
    n_replace <- length(replace_viruses)
    if (n_replace > 0) {
        cli::cli_alert_info("Replacing pattern{?s} for {n_replace} alias{?es}.")
    }

    if (sum(length(new_viruses), n_replace) == 0) {
        cli::cli_alert_info("No new alias-pattern combinations added.")
    } else {
        ## Add new aliases
        new_alias <- rbind(
            current_alias,
            data.frame(
                alias = new_viruses,
                pattern = pattern[virus == new_viruses]
            )
        )
        ## Replace patterns
        for (replacement in replace_viruses) {
            new_alias$pattern[new_alias$alias == replacement] <-
                pattern[virus == replacement]
        }

        ## Change environment
        assign("alias", new_alias, envir = pkg_env)

        ## Save to where the environment is loaded
        alias_path <- BiocFileCache::bfcquery(pkg_env$beer_cache, "alias")$rpath
        save(alias,
            envir = pkg_env,
            file = alias_path
        )
    }
}

.deleteOneAlias <- function(virus) {
    if (!virus %in% get("alias", envir = pkg_env)$alias) {
        stop("Virus does not exist in the alias database.")
    } else {
        virus_index <- which(get("alias", pkg_env)$alias == virus)
        assign("alias", pkg_env$alias[-virus_index, ], envir = pkg_env)
    }

    alias_path <- BiocFileCache::bfcquery(pkg_env$beer_cache, "alias")$rpath
    save(alias,
        envir = pkg_env,
        file = alias_path
    )
}

#' @describeIn aliases remove an alias from the database.
#' @export
deleteAlias <- function(virus) {
    for (i in virus) .deleteOneAlias(i)
}

context("Aliases for peptide subsetting can be created, modified, and deleted.")

test_that("the alias database can be accessed and modified", {

    # check getter
    expect_equal(getAlias("HIV"), "Human immunodeficiency virus")

    # check setter when virus does not exist
    setAlias("test_virus", "test_pattern")
    expect_equal(getAlias("test_virus"), "test_pattern")

    # Check setter when virus exists
    setAlias("test_virus", "test_pattern2")
    expect_equal(getAlias("test_virus"), "test_pattern2")

    # Check remove function
    deleteAlias("test_virus")
    expect_equal(getAlias("test_virus"), NA_character_)
    expect_error(
        deleteAlias("test_virus"),
        "Virus does not exist in the alias database."
    )
})

test_that("Alias functions can be applied to vectors", {
    # Check vectorization
    expect_equal(
        getAlias(c("HIV", "EBV")),
        c("Human immunodeficiency virus", "Epstein-Barr")
    )
    setAlias(c("test_1", "HIV"), c("pattern_1", "hiv"))
    expect_equal(getAlias(c("test_1", "HIV")), c("pattern_1", "hiv"))
    deleteAlias(c("test_1", "HIV"))
    expect_equal(getAlias(c("test_1", "HIV")), c(NA_character_, NA_character_))

    ## clean-up test space
    setAlias("HIV", "Human immunodeficiency virus")
})

#' @include PhIPData-class.R
NULL

#' Calculate total read counts for each sample.
#'
#' This function is a wrapper function for \link{colSums} on the \code{counts}
#' assay.
#'
#' @param object \linkS4class{PhIPData} object
#' @param ... arguments passed to \link{colSums}
#' @param withDimnames logical; if true, the vector names are the sample names;
#' otherwise the vector is unnamed.
#'
#' @return a (named) numeric vector. The length of the vector is equal to the
#' number of samples.
#'
#' @examples
#' example("PhIPData")
#' librarySize(phip_obj)
#'
#' ## Return an unnamed vector
#' librarySize(phip_obj, withDimnames = FALSE)
#' @export
librarySize <- function(object, ..., withDimnames = TRUE) {
    sums <- colSums(counts(object, ...))
    if (withDimnames) {
        sums
    } else {
        unname(sums)
    }
}

#' Proportion of sample reads
#'
#' This function calculates the proportion of total sample reads pulled by
#' each peptide.
#'
#' @param object \linkS4class{PhIPData} object
#' @param withDimnames logical; if true return a matrix with the same dimension
#' names as the original object.
#'
#' @return A (named) numeric matrix with the same dimensions as the function
#' input. Matrix values are between 0 and 1.
#'
#' @examples
#' example("PhIPData")
#' propReads(phip_obj)
#'
#' ## Return an unnamed matrix
#' propReads(phip_obj, withDimnames = FALSE)
#' @export
propReads <- function(object, withDimnames = TRUE) {
    n <- librarySize(object, withDimnames = FALSE)
    n_matrix <- matrix(rep(n, nrow(object)), nrow = nrow(object), byrow = TRUE)
    counts_matrix <- as.matrix(PhIPData::counts(object))
    prop_matrix <- counts_matrix / n_matrix
    if (withDimnames) {
        prop_matrix
    } else {
        unname(prop_matrix)
    }
}

context("Computing summaries of PhIPData objects")

# source("setup.R")

test_that("library sizes are accurately calculated", {
    expect_equal(librarySize(phip_obj), colSums(counts))
    expect_equal(
        librarySize(phip_obj, withDimnames = FALSE),
        unname(colSums(counts))
    )
})

test_that("proportion of reads pulled are accurately calculated", {
    manual_prop <- counts / matrix(rep(colSums(counts), nrow(counts)),
        nrow = nrow(counts), byrow = TRUE
    )
    expect_equal(propReads(phip_obj), manual_prop)
    expect_equal(
        propReads(phip_obj, withDimnames = FALSE),
        unname(manual_prop)
    )
})

## Simulated data to test PhIPData functions
set.seed(20210106)

## Import library
virscan_file <- system.file("extdata", "virscan.tsv", package = "PhIPData")
virscan_info <- readr::read_tsv(virscan_file,
    col_types = readr::cols(
        pep_id = readr::col_character(),
        pep_dna = readr::col_character(),
        pep_aa = readr::col_character(),
        pep_pos = readr::col_character(),
        pro_len = readr::col_double(),
        uniprot_acc = readr::col_character(),
        refseq = readr::col_character(),
        species = readr::col_character(),
        interspecies_specific = readr::col_character(),
        product = readr::col_character(),
        description = readr::col_character(),
        go = readr::col_character(),
        kegg = readr::col_character(),
        pfam = readr::col_character(),
        embl = readr::col_character(),
        interpro = readr::col_character(),
        pep_name = readr::col_character()
    )
) %>%
    as.data.frame()

n_samples <- 96L
n_peptides <- nrow(virscan_info)

## Generate data
counts <- matrix(sample(1:1e6, n_samples * n_peptides, replace = TRUE),
    nrow = n_peptides
)
logfc <- matrix(rnorm(n_samples * n_peptides, mean = 0, sd = 10),
    nrow = n_peptides
)
prob <- matrix(rbeta(n_samples * n_peptides, shape1 = 1, shape2 = 1),
    nrow = n_peptides
)

sampleInfo <- S4Vectors::DataFrame(
    sample_name = paste0("sample", 1:n_samples),
    gender = sample(c("M", "F"), n_samples,
        replace = TRUE
    ),
    group = sample(c("beads", "ART", "no ART"),
        n_samples,
        replace = TRUE
    )
)
## Set names
rownames(counts) <- rownames(logfc) <-
    rownames(prob) <- rownames(virscan_info) <-
    paste0("pep_", 1:n_peptides)

colnames(counts) <- colnames(logfc) <-
    colnames(prob) <- rownames(sampleInfo) <-
    paste0("sample_", 1:n_samples)

## default object
phip_obj <- PhIPData(
    counts = counts, logfc = logfc, prob = prob,
    sampleInfo = sampleInfo, peptideInfo = virscan_info
)

withr::defer(
    unlink(system.file(package = "PhIPData", "extdata/defaults.rda")),
    teardown_env()
)

#' @import SummarizedExperiment methods S4Vectors
#' @include defineBeads.R
NULL

### PhIPData class  ==============================================
#' The PhIPData class
#'
#' @description
#' The \code{PhIPData} class is a matrix-like container designed to organize
#' results from phage-immunoprecipitation (PhIP-Seq) experiments. Rows in
#' PhIPData objects represent peptides and columns represent samples. Each
#' object contains at least three assays:
#'
#' \itemize{
#'     \item{\code{counts}:} a matrix of raw read counts,
#'     \item{\code{logfc}:} a matrix of log2 estimated fold-change in
#'          comparison to beads-only samples,
#'     \item{\code{prob}:} a matrix of probabilities associated with whether
#'          a sample has an enriched antibody response for a peptide.
#' }
#'
#' The \code{PhIPData} class extends the
#' \linkS4class{RangedSummarizedExperiment} class, so methods documented
#' in \linkS4class{RangedSummarizedExperiment} and
#' \linkS4class{SummarizedExperiment} also work on \code{PhIPData} objects.
#'
#' @details
#' Rows of \code{PhIPData} objects correspond to peptides of interest and are
#' organized in \link{GRanges} or \link{GRangesList} objects. Though originally
#' designed for genomic ranges, the sequence name and genomic range information
#' in \link{GRanges} objects can be replaced with peptide names and amino acid
#' positions, respectively. If no peptide names are given, peptides are given
#' the names of \code{pep_rownum}. Peptide positions are specified by columns
#' \code{pos_start} and \code{pos_end} in the \code{peptideInfo} argument of the
#' constuctor. Missing position information is set to 0. Additional peptide
#' annotation can also be stored in \link{GRanges} objects and can be used
#' to subset \code{PhIPData} objects as shown below.
#'
#' Columns of \code{PhIPData} objects represent samples. Sample metadata
#' are stored in a \link{DataFrame} and can be accessed as shown below. If no
#' sample names are specified, samples are given default names of
#' \code{sample_colnum}.
#'
#' Unlike \link{RangedSummarizedExperiment}/\link{SummarizedExperiment} objects,
#' \code{PhIPData} objects must contain \code{counts}, \code{logfc},
#' \code{prob}. If any of the three assays are missing when the constructor is
#' called, an empty matrix of the same names and dimensions is initialized for
#' that assay. Sample and peptide names are harmonized across assays and
#' annotation during construction and replacement.
#'
#' Though `counts` typically contain integer values for the number of reads
#' aligned to each peptide, `PhIPData` only requires that stored values are
#' non-negative numeric values. Pseudocounts or non-integer count values can
#' also be stored in the `counts` assay.
#'
#' @section Constructor:
#' \code{PhIPData} objects are constructed using the homonymous function and
#' arguments as described above. Any \code{PhIPData} object can be created
#' so long as peptide and sample identifiers (or lack thereof) are specified
#' via any of the parameters.
#'
#' @seealso
#'      \code{\link{PhIPData-methods}} for accessors and modifiers for PhIPData
#'      components.
#'      \linkS4class{SummarizedExperiment}
.PhIPData <- setClass("PhIPData", contains = "RangedSummarizedExperiment")

### PhIPData constructor =============================================
#' @rdname PhIPData-class
#'
#' @param counts a \code{matrix}, \code{data.frame}, or
#'     \code{\linkS4class{DataFrame}}of \strong{integer} read counts.
#' @param logfc a \code{matrix}, \code{data.frame}, or
#'     \code{\linkS4class{DataFrame}} of log2 estimated fold changes.
#' @param prob a \code{matrix}, \code{data.frame}, or
#'     \code{\linkS4class{DataFrame}} of probability values (p-values or
#'     posterior probabilities) for enrichment estimates.
#' @param peptideInfo a \code{data.frame} or \code{\linkS4class{DataFrame}} of
#'    peptide information.
#' @param sampleInfo a \code{data.frame} or \code{\linkS4class{DataFrame}} of
#'     additional sample information.
#' @param metadata a \code{list} object containing experiment-specific metadata.
#' @param .defaultNames vector of names to use when sample and peptide
#'     identifiers disagree across the metadata and the \code{counts},
#'     \code{logfc}, and \code{prob} matrices. If \code{.defaultNames} is of
#'     length 1, the same source is used for both peptide and sample
#'     identifiers. If \code{.defaultNames} is longer than 2, the first and
#'     second elements correspond to the names for peptides and samples,
#'     respectively.
#'
#'     Valid options are:
#'     \itemize{
#'         \item{"info": }{names should be taken from the \code{SampleInfo} or
#'             \code{peptideInfo} objects.}
#'         \item{"counts": }{names should be taken from the row/column names
#'             of the \code{counts} object.}
#'         \item{"logfc": }{names should be taken from the row/column names of
#'             the \code{logfc} object.}
#'         \item{"prob": }{names should be taken from the row/column names of
#'             the \code{prob} object.}
#'     }
#'
#' @return A \code{PhIPData} object.
#'
#' @examples
#' ## Construct a new PhIPData object
#' counts_dat <- matrix(sample(1:1e6, 25, replace = TRUE), nrow = 5)
#' logfc_dat <- matrix(rnorm(25, 0, 10), nrow = 5)
#' prob_dat <- matrix(rbeta(25, 1, 1), nrow = 5)
#'
#' peptide_meta <- data.frame(
#'     pos_start = 1:5,
#'     pos_end = 6:10,
#'     species = c(rep("HIV", 3), rep("EBV", 2))
#' )
#' sample_meta <- data.frame(
#'     gender = sample(c("M", "F"), 5, TRUE),
#'     group = sample(c("ctrl", "trt", "beads"), 5, TRUE)
#' )
#' exp_meta <- list(
#'     date_run = as.Date("2021/01/20"),
#'     reads_per_sample = colSums(counts_dat)
#' )
#'
#' rownames(counts_dat) <- rownames(logfc_dat) <-
#'     rownames(prob_dat) <- rownames(peptide_meta) <-
#'     paste0("pep_", 1:5)
#' colnames(counts_dat) <- colnames(logfc_dat) <-
#'     colnames(prob_dat) <- rownames(sample_meta) <-
#'     paste0("sample_", 1:5)
#'
#' phip_obj <- PhIPData(
#'     counts_dat, logfc_dat, prob_dat,
#'     peptide_meta, sample_meta, exp_meta
#' )
#' phip_obj
#' @export
#' @importClassesFrom edgeR DGEList
#' @importFrom S4Vectors DataFrame isEmpty setValidity2
#' @importFrom GenomicRanges GRanges
#' @importFrom IRanges IRanges
#' @importFrom cli cli_alert_warning
PhIPData <- function(counts = matrix(nrow = 0, ncol = 0),
    logfc = matrix(nrow = 0, ncol = 0),
    prob = matrix(nrow = 0, ncol = 0),
    peptideInfo = S4Vectors::DataFrame(),
    sampleInfo = S4Vectors::DataFrame(),
    metadata = list(),
    .defaultNames = "info") {

    ## Variables defined for convenience
    assays <- c("counts", "logfc", "prob")
    assays_missing <- assays[c(missing(counts), missing(logfc), missing(prob))]

    assay_list <- list(counts = counts, logfc = logfc, prob = prob)
    .defaultNames <- if (length(.defaultNames) == 1) {
        rep(.defaultNames, 2)
    } else {
        .defaultNames
    }

    ## Check that input dimensions are matched.
    dims <- .checkDims(counts, logfc, prob, peptideInfo, sampleInfo)
    if (is.character(dims)) {
        stop(dims)
    }

    ## Get peptide names. If no peptide names are given but there are peptides,
    ## assign names of "pep_rownumber"
    peptide_names <- .getPeptideNames(
        counts, logfc, prob, peptideInfo,
        .defaultNames[1]
    )
    peptide_names <- if (is.null(peptide_names) & dims[1] != 0) {
        paste0("pep_", seq_len(dims[1]))
    } else {
        peptide_names
    }

    ## Get sample names. If no sample names are given but there are samples,
    ## assign names of "sample_colnumber"
    sample_names <- .getSampleNames(
        counts, logfc, prob, sampleInfo,
        .defaultNames[2]
    )
    sample_names <- if (is.null(sample_names) & dims[2] != 0) {
        paste0("sample_", seq_len(dims[2]))
    } else {
        sample_names
    }

    ## Set missing assays to DataFrames with dimensions and names corresponding to
    ## given matrices. All sample and peptide names are set to be identical
    ## even if they were mismatched in the inputs.
    assay_list <- .tidyAssays(
        assay_list, assays_missing, peptide_names,
        sample_names
    )

    ## Define peptide information
    tidied_pepInfo <- .tidyPeptideInfo(peptideInfo, peptide_names)
    pep_meta <- tidied_pepInfo[["pep_meta"]]
    pep_start <- tidied_pepInfo[["pep_start"]]
    pep_end <- tidied_pepInfo[["pep_end"]]

    row_info <- GenomicRanges::GRanges(
        seqnames = peptide_names,
        ranges = IRanges::IRanges(
            start = pep_start,
            end = pep_end
        )
    )
    if (!S4Vectors::isEmpty(pep_meta)) {
        mcols(row_info) <- pep_meta
    }

    ## Define sample information
    sample_meta <- if (missing(sampleInfo)) {
        S4Vectors::DataFrame(
            group = rep(NA, length = length(sample_names)),
            row.names = sample_names
        )
    } else {

        ## Add 'group' column if it is not present
        if (!"group" %in% colnames(sampleInfo)) {
            cli_alert_warning("No 'group' column in sampleInfo. Adding empty column.")
            sampleInfo$group <- NA
        }

        ## Warn if there are no beads-only samples
        if (!getBeadsName() %in% sampleInfo$group) {
            cli_alert_info("No beads-only samples present in the PhIPData object.")
        }

        S4Vectors::DataFrame(sampleInfo, row.names = sample_names)
    }

    ## Make RangedSummarizedExperiment
    se_object <- SummarizedExperiment::SummarizedExperiment(
        assays = list(
            counts = assay_list[["counts"]],
            logfc = assay_list[["logfc"]],
            prob = assay_list[["prob"]]
        ),
        rowRanges = row_info,
        colData = sample_meta,
        metadata = metadata
    )

    .PhIPData(se_object)
}

.checkDims <- function(counts, logfc, prob, peptideInfo, sampleInfo) {
    row_dims <- c(nrow(counts), nrow(logfc), nrow(prob), nrow(peptideInfo))
    col_dims <- c(ncol(counts), ncol(logfc), ncol(prob), nrow(sampleInfo))

    fixed_row <- if (length(unique(row_dims)) == 1) {
        unique(row_dims)
    } else {
        unique(row_dims[row_dims != 0])
    }
    fixed_col <- if (length(unique(col_dims)) == 1) {
        unique(col_dims)
    } else {
        unique(col_dims[col_dims != 0])
    }

    match <- c(length(fixed_row) == 1, length(fixed_col) == 1)
    error <- paste0(
        "The number of ",
        paste0(c("peptides", "samples")[!match], collapse = " and "),
        " differs across inputs."
    )

    if (sum(match) < 2) {
        error
    } else {
        c(fixed_row, fixed_col)
    }
}

.getPeptideNames <- function(counts, logfc, prob, peptideInfo, default) {
    peptide_names <- list(
        rownames(counts), rownames(logfc),
        rownames(prob), rownames(peptideInfo)
    )
    non_missing_len <- vapply(peptide_names, length, numeric(1))
    peptide_names <- unique(peptide_names[non_missing_len != 0])

    peptide_warning <- paste0(
        "Peptide names are not identical across inputs. ",
        "Using peptide names from "
    )
    default_error <- paste0(
        "Invalid '.defaultNames' supplied. Valid ",
        "'.defaultNames' options are 'info', 'counts', ",
        "'logfc', or 'prob'."
    )

    if (length(peptide_names) == 0) {
        peptide_names <- NULL
    } else if (length(peptide_names) == 1) {
        peptide_names <- peptide_names[[1]]
    } else if (default == "info") {
        cli::cli_alert_warning(paste0(peptide_warning, "'peptideInfo'."))
        peptide_names <- rownames(peptideInfo)
    } else if (default == "counts") {
        cli::cli_alert_warning(paste0(peptide_warning, "'counts'."))
        peptide_names <- rownames(counts)
    } else if (default == "logfc") {
        cli::cli_alert_warning(paste0(peptide_warning, "'logfc'."))
        peptide_names <- rownames(logfc)
    } else if (default == "prob") {
        cli::cli_alert_warning(paste0(peptide_warning, "'prob'."))
        peptide_names <- rownames(prob)
    } else {
        stop(default_error)
    }

    peptide_names
}

.getSampleNames <- function(counts, logfc, prob, sampleInfo, default) {
    sample_names <- list(
        colnames(counts), colnames(logfc),
        colnames(prob), rownames(sampleInfo)
    )
    non_missing_len <- vapply(sample_names, length, numeric(1))
    sample_names <- unique(sample_names[non_missing_len != 0])

    sample_warning <- paste0(
        "Sample names are not identical across inputs. ",
        "Using sample names from "
    )
    default_error <- paste0(
        "Invalid '.defaultNames' supplied. Valid ",
        "'.defaultNames' options are: 'info', 'counts', ",
        "'logfc', or 'prob'."
    )

    if (length(sample_names) == 0) {
        sample_names <- NULL
    } else if (length(sample_names) == 1) {
        sample_names <- sample_names[[1]]
    } else if (default == "info") {
        cli::cli_alert_warning(paste0(sample_warning, "'sampleInfo'."))
        sample_names <- rownames(sampleInfo)
    } else if (default == "counts") {
        cli::cli_alert_warning(paste0(sample_warning, "'counts'."))
        sample_names <- colnames(counts)
    } else if (default == "logfc") {
        cli::cli_alert_warning(paste0(sample_warning, "'logfc'"))
        sample_names <- colnames(logfc)
    } else if (default == "prob") {
        cli::cli_alert_warning(paste0(sample_warning, "'prob'."))
        sample_names <- colnames(prob)
    } else {
        stop(default_error)
    }

    sample_names
}

.tidyAssays <- function(assay_list, assays_missing,
    peptide_names, sample_names) {
    assays <- c("counts", "logfc", "prob")
    assays_present <- assays[!assays %in% assays_missing]
    num_missing <- length(assays_missing)

    ## Create empty matrix missing assays
    empty_mat <- matrix(
        nrow = length(peptide_names),
        ncol = length(sample_names)
    )
    rownames(empty_mat) <- peptide_names
    colnames(empty_mat) <- sample_names
    for (assay in assays_missing) {
        assay_list[[assay]] <- empty_mat
    }

    ## Coerce all assays into matrices
    assay_list <- lapply(assay_list, as.matrix)

    ## Correct names
    for (assay in assays) {
        if (!is.null(peptide_names)) {
            rownames(assay_list[[assay]]) <- peptide_names
        }
        if (!is.null(sample_names)) {
            colnames(assay_list[[assay]]) <- sample_names
        }
    }

    assay_list
}

.tidyPeptideInfo <- function(peptideInfo, peptide_names) {
    peptideInfo <- DataFrame(peptideInfo)

    if (all(c(dim(peptideInfo), length(peptide_names)) == 0)) {
        pep_meta <- S4Vectors::DataFrame(row.names = peptide_names)
        pep_start <- rep(0, length(peptide_names))
        pep_end <- rep(0, length(peptide_names))
    } else {
        pep_meta <- S4Vectors::DataFrame(peptideInfo[, !colnames(peptideInfo) %in%
            c("pos_start", "pos_end"),
        drop = FALSE
        ])

        warnings <- c(
            no_start = (!"pos_start" %in% colnames(peptideInfo)),
            no_end = (!"pos_end" %in% colnames(peptideInfo))
        )
        warnings["missing_start"] <- if (!warnings["no_start"]) {
            any(is.na(peptideInfo[, "pos_start"]))
        } else {
            TRUE
        }
        warnings["missing_end"] <- if (!warnings["no_end"]) {
            any(is.na(peptideInfo[, "pos_end"]))
        } else {
            TRUE
        }

        msg <- if (sum(warnings[c("no_start", "no_end")]) > 0) {
            paste0(
                "Missing peptide ",
                paste(c("start", "end")[warnings[c("no_start", "no_end")]],
                    collapse = " and "
                ),
                " position information."
            )
        } else {
            NULL
        }

        msg <- if (any(warnings)) {
            paste0(msg, " Replacing missing values with 0.")
        } else {
            msg
        }
        if (length(msg) > 0) {
            cli::cli_alert_warning(msg)
        }

        ## Add pos_start and pos_end columns if not present, set to 0 if missing
        if (!all(c("pos_start", "pos_end") %in% colnames(peptideInfo))) {
            pep_start <- rep(0, length(peptide_names))
            pep_end <- rep(0, length(peptide_names))
        } else {
            pep_start <- if ("pos_start" %in% colnames(peptideInfo)) {
                replace(
                    peptideInfo[["pos_start"]],
                    is.na(peptideInfo[, "pos_start"]), 0
                )
            } else {
                rep(0, length(peptide_names))
            }

            pep_end <- if ("pos_end" %in% colnames(peptideInfo)) {
                replace(peptideInfo[["pos_end"]], is.na(peptideInfo[, "pos_end"]), 0)
            } else {
                rep(0, length(length(peptide_names)))
            }
        }
    }

    list(
        pep_meta = pep_meta,
        pep_start = pep_start,
        pep_end = pep_end
    )
}

### Validity ==============================================
## 1. `counts`, `logfc`, and `prob` must be matrices in a valid PhIPData object.
.checkAssays <- function(x) {
    assay_list <- c("counts", "logfc", "prob")

    if (!all(assay_list %in% names(assays(x)))) {
        missing_assays <- assay_list[!assay_list %in% names(assays(x))]
        paste0(
            "`counts`, `logfc`, and `prob` assays must be included ",
            "in a PhIPData object. The following assays are missing: ",
            paste(missing_assays, collapse = ", "), "."
        )
    } else {
        NULL
    }
}

## 2. counts cannot have negative entries
.checkCounts <- function(x) {
    error <- character()

    if (all(dim(x) != 0)) {
        counts_pos <- (counts(x) >= 0) | is.na(counts(x))
        if (!all(counts_pos)) {
            msg <- "cannot have negative entries"
            error <- c(error, msg)
        }

        error <- paste0(error, collapse = " and ")

        if (error != "") {
            paste0("'counts' ", error, ".")
        } else {
            NULL
        }
    } else {
        NULL
    }
}

## 3. Sample and peptide dimensions must be the same within the object.
.checkObjectDim <- function(x) {
    dim_check <- .checkDims(
        counts(x), logfc(x), prob(x),
        peptideInfo(x), sampleInfo(x)
    )
    if (is.character(dim_check)) {
        dim_check
    } else {
        NULL
    }
}

## 4. sample and peptide names must be identical across all assays
## and annotation information.
.checkNames <- function(x) {
    sample_names <- list(
        colnames(counts(x)), colnames(logfc(x)),
        colnames(prob(x)), rownames(sampleInfo(x))
    )
    peptide_names <- list(
        rownames(counts(x)), rownames(logfc(x)),
        rownames(prob(x)), names(peptideInfo(x))
    )

    match <- c(
        length(unique(sample_names)) == 1,
        length(unique(peptide_names)) == 1
    )

    error <- paste0(
        "Names do not match across ",
        paste0(c("samples", "peptides")[!match], collapse = " and "),
        "."
    )

    if (sum(match) < 2) {
        error
    } else {
        NULL
    }
}

.checkSampleInfo <- function(x) {
    if (!"group" %in% colnames(sampleInfo(x))) {
        "'group' column must be present in sample information."
    } else {
        NULL
    }
}

.validPhIPData <- function(x) {
    if (!isEmpty(x)) {
        c(
            .checkAssays(x), .checkCounts(x), .checkObjectDim(x), .checkNames(x),
            .checkSampleInfo(x)
        )
    }
}

S4Vectors::setValidity2("PhIPData", .validPhIPData)

### Getters ==============================================
#' @name PhIPData-methods
#' @title Accessing and Modifying Information in PhIPData objects
#'
#' @description Methods to extract and modify \code{assay(s)}(including
#' convenient functions for \code{counts}, \code{logfc}, and \code{prob}),
#' \code{sampleInfo}, \code{peptideInfo}, and \code{metadata}.
#'
#' @details In addition to the functions detailed in
#' \linkS4class{RangedSummarizedExperiment}, the \code{PhIPData} class includes
#' conveniently named functions to quickly access and modify frequently used
#' components of PhIPData objects.
#'
#' Replacement functions ensure that names of the replacement object are matched
#' with the names of the \code{PhIPData} object.
#'
#' Since packages for identifying differential expression in RNA-seq experiments
#' are frequently used for estimating fold-changes for peptide enrichments,
#' the class also includes coercion methods to and from \linkS4class{DGEList}s.
#'
#' @section Available methods:
#' In the following code snippets, \code{x} is a \linkS4class{PhIPData} object,
#' \code{value} is a matrix-like object with the same dimensions as \code{x},
#' and \code{...} are further arguments passed to \code{\link{assay}}
#' (for the getter) or \code{\link{assay<-}} (for the setter).
#' \describe{
#' \item{\code{counts(x, ...)}, \code{counts(x, ...) <- value}:}{
#' Get or set a matrix of raw read counts
#' }
#' \item{\code{logfc(x, ...)}, \code{logfc(x, ...) <- value}:}{
#' Get or set a matrix of log2 estimated fold changes (in comparison to
#' beads-only samples)
#' }
#' \item{\code{prob(x, ...)}, \code{pob(x, ...) <- value}:}{
#' Get or set a matrix of probabilities associated with whether
#'          a sample has an enriched antibody response for a peptide.
#' }
#' }
#'
#' @examples
#' example("PhIPData")
#'
#' replacement_dat <- matrix(1L, nrow = 5, ncol = 5)
#'
#' ## SummarizedExperiment Accessors and Setters
#' assays(phip_obj)
#' assays(phip_obj)$counts <- replacement_dat
#' assay(phip_obj, "logfc")
#' assay(phip_obj, "logfc") <- replacement_dat
#'
#' ## counts
#' counts(phip_obj)
#' counts(phip_obj) <- counts_dat
#'
#' ## logfc
#' logfc(phip_obj)
#' logfc(phip_obj) <- logfc_dat
#'
#' ## prob
#' prob(phip_obj)
#' prob(phip_obj) <- replacement_dat
#'
#' ## coercion functions
#' as(phip_obj, "DGEList")
#' as(phip_obj, "List")
#' as(phip_obj, "list")
#' @param object A \code{PhIPData} object
#' @param x A \code{PhIPData} object
#' @param i A \code{numeric}, {character}
#' @param withDimnames Parameter for
#'      \linkS4class{RangedSummarizedExperiment} class functions. Overrided
#'      since row/column names are automatically synced within each object.
#' @param ... parameters for \code{\link[SummarizedExperiment]{assays}},
#'      which are typically not needed.
#' @param value A \code{matrix}, \code{data.frame}, or \linkS4class{DataFrame}
#'      of the same dimensions (not necessarily the same names)
#'
#' @return Accessors: a \link{DataFrame} object
#' @return Setters: a \code{PhIPData} object
#'
#' @seealso
#' \code{\link[SummarizedExperiment]{assays}} for
#' \linkS4class{SummarizedExperiment} operations.
NULL

#' @export
#' @rdname PhIPData-methods
#' @importFrom BiocGenerics counts
setMethod("counts", "PhIPData", function(object, ...) {
    SummarizedExperiment::assay(object, "counts", ...)
})

#' @export
#' @rdname PhIPData-methods
setGeneric("logfc", function(object, ...) standardGeneric("logfc"))

#' @export
#' @rdname PhIPData-methods
setMethod("logfc", "PhIPData", function(object, ...) {
    SummarizedExperiment::assay(object, "logfc", ...)
})


#' @export
#' @rdname PhIPData-methods
setGeneric("prob", function(object, ...) standardGeneric("prob"))

#' @export
#' @rdname PhIPData-methods
setMethod("prob", "PhIPData", function(object, ...) {
    SummarizedExperiment::assay(object, "prob", ...)
})

#' @export
#' @rdname PhIPData-methods
setGeneric("peptideInfo", function(object, ...) standardGeneric("peptideInfo"))

#' @export
#' @rdname PhIPData-methods
setMethod("peptideInfo", "PhIPData", function(object, ...) {
    SummarizedExperiment::rowRanges(object, ...)
})


#' @export
#' @rdname PhIPData-methods
setGeneric("sampleInfo", function(object, ...) standardGeneric("sampleInfo"))

#' @export
#' @rdname PhIPData-methods
setMethod("sampleInfo", "PhIPData", function(object, ...) {
    SummarizedExperiment::colData(object, ...)
})

### Setters ==============================================
# This `assays` and `assay` replacement functions differs from the
# SummarizedExperiment assays functions in that mismatched names returns
# a valid object rather than an error.
#' @export
#' @rdname PhIPData-methods
setReplaceMethod("assays", c("PhIPData", "list"), function(x, ..., value) {
    pep_names <- rownames(x)
    sample_names <- colnames(x)

    value <- if (length(value) == 1) {
        rownames(value) <- pep_names
        colnames(value) <- sample_names

        as.matrix(value)
    } else if (length(value) > 1) {
        lapply(value, function(assay) {
            rownames(assay) <- pep_names
            colnames(assay) <- sample_names

            as.matrix(assay)
        })
    } else {
        value
    }

    new_object <- callNextMethod()

    # Ensure that counts, logfc, and probs are in the final object
    error <- .checkAssays(new_object)
    if (length(error)) {
        stop(error)
    }

    validObject(new_object)
    new_object
})

#' @export
#' @rdname PhIPData-methods
setReplaceMethod(
    "assays", c("PhIPData", "SimpleList"),
    function(x, ..., value) {
        pep_names <- rownames(x)
        sample_names <- colnames(x)

        value <- if (length(value) == 1) {
            rownames(x) <- pep_names
            colnames(x) <- sample_names

            as.matrix(value)
        } else if (length(value) > 1) {
            lapply(value, function(assay) {
                rownames(assay) <- pep_names
                colnames(assay) <- sample_names

                as.matrix(assay)
            })
        } else {
            value
        }

        new_object <- callNextMethod()

        # Ensure that counts, logfc, and probs are in the final object
        error <- .checkAssays(new_object)
        if (length(error)) {
            stop(error)
        }

        validObject(new_object)
        new_object
    }
)

#' @export
#' @rdname PhIPData-methods
setReplaceMethod(
    "assay", c("PhIPData", "missing"),
    function(x, i, withDimnames = TRUE, ..., value) {
        new_object <- if (!is.null(value)) {
            rownames(value) <- rownames(x)
            colnames(value) <- colnames(x)

            callNextMethod()
        } else {
            remaining_assays <- setdiff(assayNames(x), "counts")
            assays(x) <- assays(x)[remaining_assays]

            x
        }

        # Ensure that counts, logfc, and probs are in the final object
        error <- .checkAssays(new_object)
        if (length(error)) {
            stop(error)
        }

        validObject(new_object)
        new_object
    }
)

#' @export
#' @rdname PhIPData-methods
setReplaceMethod(
    "assay", c("PhIPData", "numeric"),
    function(x, i, withDimnames = TRUE, ..., value) {
        new_object <- if (!is.null(value)) {
            rownames(value) <- rownames(x)
            colnames(value) <- colnames(x)

            callNextMethod()
        } else {
            remaining_assays <- assayNames(x)[-i]
            assays(x) <- assays(x)[remaining_assays]

            x
        }

        # Ensure that counts, logfc, and probs are in the final object
        error <- .checkAssays(new_object)
        if (length(error)) {
            stop(error)
        }

        validObject(new_object)
        new_object
    }
)

#' @export
#' @rdname PhIPData-methods
setReplaceMethod(
    "assay", c("PhIPData", "character"),
    function(x, i, withDimnames = TRUE, ..., value) {
        new_object <- if (!is.null(value)) {
            rownames(value) <- rownames(x)
            colnames(value) <- colnames(x)

            callNextMethod()
        } else {
            remaining_assays <- setdiff(assayNames(x), i)
            assays(x) <- assays(x)[remaining_assays]

            x
        }

        # Ensure that counts, logfc, and probs are in the final object
        error <- .checkAssays(new_object)
        if (length(error)) {
            stop(error)
        }

        validObject(new_object)
        new_object
    }
)

# Convenience functions for standard Assays
#' @export
#' @rdname PhIPData-methods
#' @importFrom BiocGenerics "counts<-"
setReplaceMethod("counts", c("PhIPData", "ANY"), function(object, ..., value) {
    assay(object, "counts") <- value
    object
})

#' @export
#' @rdname PhIPData-methods
setGeneric("logfc<-", function(object, ..., value) standardGeneric("logfc<-"))

#' @export
#' @rdname PhIPData-methods
setReplaceMethod("logfc", "PhIPData", function(object, ..., value) {
    assay(object, "logfc") <- value
    object
})


#' @export
#' @rdname PhIPData-methods
setGeneric("prob<-", function(object, ..., value) standardGeneric("prob<-"))

#' @export
#' @rdname PhIPData-methods
setReplaceMethod("prob", "PhIPData", function(object, ..., value) {
    assay(object, "prob") <- value
    object
})

#' @export
#' @rdname PhIPData-methods
setGeneric("peptideInfo<-", function(object, value) {
    standardGeneric("peptideInfo<-")
})

#' @export
#' @rdname PhIPData-methods
setReplaceMethod("peptideInfo", "PhIPData", function(object, value) {
    # check # of peptides match with assays
    if (nrow(value) != nrow(counts(object))) {
        msg <- paste0(
            "The number of peptides in the annotation differ from ",
            "the number of peptides in `counts`."
        )
        stop(msg)
    }

    # get peptide names and sample names from existing x
    rownames(value) <- peptide_names <- dimnames(object)[[1]]

    new_pepInfo <- .tidyPeptideInfo(value, peptide_names)

    pep_meta <- new_pepInfo[["pep_meta"]]
    pep_start <- new_pepInfo[["pep_start"]]
    pep_end <- new_pepInfo[["pep_end"]]

    row_info <- GenomicRanges::GRanges(
        seqnames = peptide_names,
        ranges = IRanges::IRanges(
            start = pep_start,
            end = pep_end
        )
    )
    if (!S4Vectors::isEmpty(pep_meta)) {
        mcols(row_info) <- pep_meta
    }

    # Why does setting a new rowRanges erase all my row names for the x?!?!?!?!?
    rowRanges(object) <- row_info
    rownames(object) <- peptide_names

    validObject(object)

    object
})

#' @export
#' @rdname PhIPData-methods
setGeneric("sampleInfo<-", function(object, ..., value) {
    standardGeneric("sampleInfo<-")
})

#' @export
#' @rdname PhIPData-methods
#' @importFrom cli cli_alert_warning cli_alert_info
setReplaceMethod("sampleInfo", "PhIPData", function(object, value) {
    # check # of samples match with assays
    if (nrow(value) != ncol(counts(object))) {
        msg <- paste0(
            "The number of samples in the annotation differ from ",
            "the number of samples in `counts`."
        )
        stop(msg)
    }

    # get peptide names and sample names from existing object
    rownames(value) <- sample_names <- dimnames(object)[[2]]

    sample_meta <- S4Vectors::DataFrame(value, row.names = sample_names)

    ## Add 'group' column if it is not present
    if (!"group" %in% colnames(sample_meta)) {
        cli::cli_alert_warning(paste0(
            "No 'group' column in sampleInfo. ",
            "Adding empty column."
        ))
        sample_meta$group <- NA
    }

    ## Warn if there are no beads-only samples
    if (!getBeadsName() %in% sample_meta$group) {
        cli::cli_alert_info(paste0(
            "No beads-only samples present in the ",
            "PhIPData object."
        ))
    }

    colData(object) <- sample_meta

    validObject(object)

    object
})

### Subsetters ==============================================

### Generics ==============================================
setMethod("show", "PhIPData", function(object) {
    callNextMethod()
    beads_name <- getBeadsName()
    num_beads <- sum(sampleInfo(object)$group == beads_name, na.rm = TRUE)
    cat("beads-only name(", num_beads, "): ", beads_name,
        sep = ""
    )
})

setMethod("isEmpty", "PhIPData", function(x) {
    isEmpty(counts(x)) & isEmpty(logfc(x)) &
        isEmpty(prob(x)) & isEmpty(peptideInfo(x)) &
        isEmpty(sampleInfo(x))
})

### Coercion methods ==============================================
# DGEList to PhIPData
setAs("DGEList", "PhIPData", function(from) {
    count_mat <- if (is.null(from[["counts"]])) {
        matrix(nrow = 0, ncol = 0)
    } else {
        from[["counts"]]
    }
    sample_info <- if (is.null(from[["samples"]])) {
        matrix(nrow = 0, ncol = 0)
    } else {
        from[["samples"]]
    }

    pep_info <- if (is.null(from[["genes"]])) {
        S4Vectors::DataFrame()
    } else {
        from[["genes"]]
    }

    PhIPData(counts = count_mat, peptideInfo = pep_info, sampleInfo = sample_info)
})

# PhIPData to DGEList
setAs("PhIPData", "DGEList", function(from) {
    count_mat <- counts(from)
    sample_info <- sampleInfo(from)
    peptide_info <- peptideInfo(from)
    group_labs <- if ("group" %in% colnames(sample_info)) {
        sample_info$group
    } else {
        NULL
    }

    edgeR::DGEList(
        counts = count_mat,
        samples = sample_info[colnames(sample_info) != "group"],
        group = group_labs,
        genes = peptide_info
    )
})

# PhIPData to list
setAs("PhIPData", "list", function(from) {
    list(
        assays = assays(from),
        peptideInfo = peptideInfo(from),
        sampleInfo = sampleInfo(from),
        metadata = metadata(from)
    )
})

setAs("list", "PhIPData", function(from) {
    PhIPData(
        counts = from$assays[["counts"]],
        logfc = from$assays[["logfc"]],
        prob = from$assays[["prob"]],
        peptideInfo = from$peptideInfo,
        sampleInfo = from$sampleInfo,
        metadata = from$metadata
    )
})


# PhIPData to List
setAs("PhIPData", "List", function(from) {
    List(
        assays = assays(from),
        peptideInfo = peptideInfo(from),
        sampleInfo = sampleInfo(from),
        metadata = metadata(from)
    )
})

setAs("List", "PhIPData", function(from) {
    PhIPData(
        counts = from[["assays"]][["counts"]],
        logfc = from[["assays"]][["logfc"]],
        prob = from[["assays"]][["prob"]],
        peptideInfo = from[["peptideInfo"]],
        sampleInfo = from[["sampleInfo"]],
        metadata = from[["metadata"]]
    )
})

# PhIPData to DataFrame
setAs("PhIPData", "DataFrame", function(from) {
    if (length(metadata(from)) != 0) {
        cli::cli_alert_warning("Metadata will be lost during coercion.")
    }
    n_samples <- ncol(from)
    n_peps <- nrow(from)

    ## Tidy assays
    assay_df <- DataFrame(lapply(assays(from), as.vector))

    ## Tidy sample_info
    sample_df <- rep(cbind(sample = colnames(from), sampleInfo(from)),
        each = n_peps
    )
    ## Tidy peptide_info
    peptide_df <- rep(cbind(
        DataFrame(
            peptide = rownames(from),
            pos_start = start(peptideInfo(from)),
            pos_end = end(peptideInfo(from))
        ),
        mcols(peptideInfo(from))
    ),
    times = n_samples
    )

    cbind(sample_df, peptide_df, assay_df)
})

context("Base PhIPData API functions work as expected.")

# source("setup.R")

# Test missing params ----------------------------------------
test_that("valid PhIPData objects are created when there are missing inputs.", {

    # No assay, no metadata
    expect_is(PhIPData(), "PhIPData")

    # No assay, only metadata
    expect_is(PhIPData(peptideInfo = virscan_info), "PhIPData")
    expect_is(PhIPData(sampleInfo = sampleInfo), "PhIPData")

    # only one assay/input is present, no metadata
    expect_is(PhIPData(counts = counts), "PhIPData")
    expect_is(PhIPData(logfc = logfc), "PhIPData")
    expect_is(PhIPData(prob = prob), "PhIPData")

    # one assay and metadata
    expect_is(PhIPData(counts = counts, sampleInfo = sampleInfo), "PhIPData")
    expect_is(PhIPData(counts = counts, peptideInfo = virscan_info), "PhIPData")
    expect_is(PhIPData(
        counts = counts, peptideInfo = virscan_info,
        sampleInfo = sampleInfo
    ), "PhIPData")

    # only two assays are present, no metadata
    expect_is(PhIPData(counts = counts, logfc = logfc), "PhIPData")
    expect_is(PhIPData(counts = counts, prob = prob), "PhIPData")
    expect_is(PhIPData(logfc = logfc, prob = prob), "PhIPData")

    # only two assays and metadata
    expect_is(PhIPData(
        counts = counts, logfc = logfc,
        sampleInfo = sampleInfo
    ), "PhIPData")
    expect_is(PhIPData(
        counts = counts, logfc = logfc,
        peptideInfo = virscan_info
    ), "PhIPData")
    expect_is(
        PhIPData(
            counts = counts, logfc = logfc,
            peptideInfo = virscan_info, sampleInfo = sampleInfo
        ),
        "PhIPData"
    )

    # all 3 assays, no metadata
    expect_is(PhIPData(counts = counts, logfc = logfc, prob = prob), "PhIPData")

    # all 3 assays and metadata
    expect_is(PhIPData(
        counts = counts, logfc = logfc, prob = prob,
        sampleInfo = sampleInfo
    ), "PhIPData")
    expect_is(PhIPData(
        counts = counts, logfc = logfc, prob = prob,
        peptideInfo = virscan_info
    ), "PhIPData")
    expect_is(
        PhIPData(
            counts = counts, logfc = logfc, prob = prob,
            peptideInfo = virscan_info, sampleInfo = sampleInfo
        ),
        "PhIPData"
    )

    # peptide data is missing positional information
    nopos_pepInfo <- dplyr::select(virscan_info, -contains("pos"))
    expect_is(
        PhIPData(
            counts = counts, logfc = logfc, prob = prob,
            peptideInfo = nopos_pepInfo, sampleInfo = sampleInfo
        ),
        "PhIPData"
    )
})

# Test name-fixing code ----------------------------------------
# Function to quickly compare names
expect_names <- function(object, list) {
    names <- dimnames(object)
    return(all(names[[1]] == list[[1]]) & all(names[[2]] == list[[2]]))
}

test_that("dimension names are set correctly when inputs are mismatched", {
    assay_list <- list(counts = counts, logfc = logfc, prob = prob)

    for (assay in c("counts", "prob")) {
        rownames(assay_list[[assay]]) <-
            paste0(assay, "_", rownames(assay_list[[assay]]))
        colnames(assay_list[[assay]]) <-
            paste0(assay, "_", colnames(assay_list[[assay]]))
    }
    colnames(assay_list[["logfc"]]) <-
        paste0("logfc_", colnames(assay_list[["logfc"]]))

    # Check defaults work
    expect_true(expect_names(
        PhIPData(
            assay_list[["counts"]],
            assay_list[["logfc"]],
            assay_list[["prob"]],
            virscan_info,
            sampleInfo
        ),
        list(
            rownames(peptideInfo),
            rownames(sampleInfo)
        )
    ))

    # Check single default input works as expected
    expect_true(expect_names(
        PhIPData(assay_list[["counts"]],
            assay_list[["logfc"]],
            assay_list[["prob"]],
            virscan_info,
            sampleInfo,
            .defaultNames = "counts"
        ),
        dimnames(assay_list[["counts"]])
    ))

    expect_true(expect_names(
        PhIPData(assay_list[["counts"]],
            assay_list[["logfc"]],
            assay_list[["prob"]],
            virscan_info,
            sampleInfo,
            .defaultNames = "logfc"
        ),
        dimnames(assay_list[["logfc"]])
    ))

    expect_true(expect_names(
        PhIPData(assay_list[["counts"]],
            assay_list[["logfc"]],
            assay_list[["prob"]],
            virscan_info,
            sampleInfo,
            .defaultNames = "prob"
        ),
        dimnames(assay_list[["prob"]])
    ))

    # check double default and extra default input do not generate errors
    expect_true(expect_names(
        PhIPData(assay_list[["counts"]],
            assay_list[["logfc"]],
            assay_list[["prob"]],
            virscan_info,
            sampleInfo,
            .defaultNames = c(
                "counts", "logfc",
                "extraneous"
            )
        ),
        list(
            rownames(assay_list[["counts"]]),
            colnames(assay_list[["logfc"]])
        )
    ))
})

# Test error-causing inputs ----------------------------------------
test_that("invalid inputs return proper errors", {

    # negative counts
    expect_error(
        PhIPData(counts = matrix(-15:14, nrow = 5)),
        "'counts' cannot have negative entries."
    )

    # incompatible dimensions
    expect_error(
        PhIPData(
            counts = counts,
            logfc = DataFrame(matrix(nrow = 5, ncol = 5))
        ),
        "The number of .* differs across inputs"
    )

    # invalid default behavior for mismatched names
    # (only a problem names are mismatched)
    rownames(logfc) <- paste0("logfc_", rownames(logfc))
    expect_error(
        PhIPData(counts, logfc, prob,
            virscan_info, sampleInfo,
            .defaultNames = "test"
        ),
        "Invalid '.defaultNames' supplied."
    )

    colnames(logfc) <- paste0("logfc_", colnames(logfc))
    expect_error(
        PhIPData(counts, logfc, prob,
            virscan_info, sampleInfo,
            .defaultNames = "test"
        ),
        "Invalid '.defaultNames' supplied."
    )
})

# Test getters ----------------------------------------
test_that("getter functions return objects of the expected class", {
    phip_obj <- PhIPData(
        counts = counts, logfc = logfc, prob = prob,
        sampleInfo = sampleInfo, peptideInfo = virscan_info
    )

    expect_is(counts(phip_obj), "matrix")
    expect_is(logfc(phip_obj), "matrix")
    expect_is(prob(phip_obj), "matrix")
    expect_is(peptideInfo(phip_obj), "GRanges")
    expect_is(sampleInfo(phip_obj), "DataFrame")
})

# Test setters ----------------------------------------
# 1. Check that replacement works and
# 2. Replacement fixes mismatched sample/peptide names.
test_that("setter functions change the object as desired", {
    phip_obj <- PhIPData(
        counts = counts, logfc = logfc, prob = prob,
        sampleInfo = sampleInfo, peptideInfo = virscan_info
    )

    # Check assay replacement; newinfo has different names
    replacement_matrix <- matrix(sample(1:1e6, n_samples * n_peptides,
        replace = TRUE
    ), nrow = n_peptides)

    expect_error(
        assays(phip_obj) <- list(
            assay_1 = replacement_matrix,
            assay_2 = replacement_matrix
        ),
        paste0(
            "`counts`, `logfc`, and `prob` assays must be included ",
            "in a PhIPData object. The following assays are ",
            "missing: counts, logfc, prob."
        )
    )
    assays(phip_obj) <- list(
        counts = replacement_matrix,
        logfc = replacement_matrix,
        prob = replacement_matrix
    )
    expect_equal(unname(counts(phip_obj)), replacement_matrix)
    expect_equal(unname(logfc(phip_obj)), replacement_matrix)
    expect_equal(unname(prob(phip_obj)), replacement_matrix)

    assay(phip_obj) <- counts
    expect_equal(counts(phip_obj), counts)
    assay(phip_obj, 2) <- logfc
    expect_equal(logfc(phip_obj), logfc)
    assay(phip_obj, "prob") <- prob
    expect_equal(prob(phip_obj), prob)

    counts(phip_obj) <- logfc(phip_obj) <- prob(phip_obj) <- replacement_matrix
    expect_equal(unname(counts(phip_obj)), replacement_matrix)
    expect_equal(unname(logfc(phip_obj)), replacement_matrix)
    expect_equal(unname(prob(phip_obj)), replacement_matrix)

    # Check that invalid replacement generates warning
    expect_error(
        counts(phip_obj) <- matrix(-1L, nrow = n_peptides, ncol = n_samples),
        "'counts' cannot have negative entries."
    )

    # Check peptideInfo replacement; new info has different rownames
    peptideInfo(phip_obj) <- virscan_info[, 1:10]
    pep_tidied <- .tidyPeptideInfo(
        virscan_info[, 1:10],
        dimnames(phip_obj)[[1]]
    )
    pep_expect <- GenomicRanges::GRanges(
        seqnames = dimnames(phip_obj)[[1]],
        ranges = IRanges::IRanges(
            start = pep_tidied[["pep_start"]],
            end = pep_tidied[["pep_end"]]
        )
    )

    mcols(pep_expect) <- pep_tidied[["pep_meta"]]

    expect_equal(unname(peptideInfo(phip_obj)), pep_expect)

    # Check sampleInfo replacement, new info has different rownames
    sampleInfo$ART <- sample(c("yes", "no"), n_samples, replace = TRUE)
    sampleInfo(phip_obj) <- sampleInfo

    expect_equal(sampleInfo(phip_obj), sampleInfo)
})

test_that("assays can be added and removed", {
    phip_obj <- PhIPData(
        counts = counts, logfc = logfc, prob = prob,
        sampleInfo = sampleInfo, peptideInfo = virscan_info
    )

    replacement_matrix <- matrix(runif(n_samples * n_peptides, min = 1, max = 1e6),
        nrow = n_peptides
    )

    expect_error(
        assay(phip_obj, "counts") <- NULL,
        paste0(
            "`counts`, `logfc`, and `prob` assays must be included ",
            "in a PhIPData object. The following assays are ",
            "missing: counts"
        )
    )
    assay(phip_obj, "new_assay") <- replacement_matrix
    expect_equal(
        unname(assay(phip_obj, "new_assay")),
        replacement_matrix
    )
    assay(phip_obj, "new_assay") <- NULL
    expect_true(!"new_assay" %in% assayNames(phip_obj))
})

# Test coercion functions -----------
test_that("coercion functions work as expected", {
    phip_obj <- PhIPData(
        counts = counts, logfc = logfc, prob = prob,
        sampleInfo = sampleInfo, peptideInfo = virscan_info
    )

    expect_is(as(phip_obj, "list"), "list")
    expect_is(as(phip_obj, "List"), "List")
    expect_is(as(phip_obj, "DGEList"), "DGEList")
    expect_is(as(phip_obj, "DataFrame"), "DataFrame")

    expect_is(as(as(phip_obj, "list"), "PhIPData"), "PhIPData")
    expect_is(as(as(phip_obj, "List"), "PhIPData"), "PhIPData")
    expect_is(as(as(phip_obj, "DGEList"), "PhIPData"), "PhIPData")
})

context("Check PhIPData subsetting wrappers")

# source("setup.R")

test_that("beads-only subsetting works", {
    # empty case
    expect_is(subsetBeads(PhIPData()), "PhIPData")

    # full case
    phip_obj <- PhIPData(
        counts = counts, logfc = logfc, prob = prob,
        sampleInfo = sampleInfo, peptideInfo = virscan_info
    )
    beads_sub <- subsetBeads(phip_obj)
    expect_equal(
        dim(beads_sub),
        c(nrow(phip_obj), sum(sampleInfo$group == "beads"))
    )
    expect_equal(rownames(phip_obj), rownames(beads_sub))
    expect_equal(
        colnames(phip_obj)[sampleInfo$group == "beads"],
        colnames(beads_sub)
    )
})

#' @include PhIPData-class.R
NULL

#' Subset beads-only samples
#'
#' Function to subset PhIP-seq data for beads-only samples.
#'
#' @param object \linkS4class{PhIPData} object
#' @return a \linkS4class{PhIPData} object.
#'
#' @examples
#' example("PhIPData")
#' subsetBeads(phip_obj)
#' @export
subsetBeads <- function(object) {
    object[, object$group == getBeadsName()]
}

---
title: "PhIPData: A Container for PhIP-Seq Experiments"
author: "Athena Chen, Rob Scharpf, and Ingo Ruczinski"
date: "`r format(Sys.time(), '%B %d, %Y')`"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{PhIPData: A Container for PhIP-Seq Experiments}
  %\VignetteEncoding{UTF-8}
output:
  BiocStyle::html_document:
    number_sections: yes
    toc: true
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  comment = "##"
)
```

# Installation

We recommend installing the stable release version of `PhIPData` in Bioconductor.
This can be done using `BiocManager`:

```{r, eval = FALSE}
if (!require("BiocManager"))
    install.packages("BiocManager")
    
BiocManager::install("PhIPData")
```

To load the package:

```{r, include = TRUE, results = "hide", message = FALSE, warning = FALSE}
library(PhIPData)
```

# Introduction

The `PhIPData` class is used to store experimental results from phage-immunoprecipitation sequencing (PhIP-set) experiments in a matrix-like container.

Building on the [`RangedSummarizedExperiment`](https://bioconductor.org/packages/release/bioc/html/SummarizedExperiment.html) class, `PhIPData` contains all of functionality of `SummarizedExperiments` and includes additional operations to facilitate analysis with PhIP-seq data. Like `SummarizedExperiments`, a key feature of `PhIPData` is the coordination of metadata when subsetting `PhIPData` objects. For example, if you wanted to examine experimental data for peptides from one particular virus, you can subset the experimental data and the associated peptide annotation with one command. This ensures all metadata (for samples, peptides, etc.) remain synced with the experimental data throughout analysis. 

# Components of a `PhIPData` Object

As reflected in the figure below, the structure of a `PhIPData` object is nearly identical to the structure of a `SummarizedExperiment`/`RangedSummarizedExperiment` object. 

Each object contains at least three assays of data. These assays are:

* `counts`: matrix of raw read counts,
* `logfc`: matrix of log2 estimated fold-changes (in comparison to negative control samples),
* `prob`: matrix of probabilities (p-values or posterior probabilities) associated with whether a sample shows an enriched antibody response to the particular peptide. 

Though `counts` typically contain integer values for the number of reads aligned to each peptide, `PhIPData` only requires that stored values are non-negative numeric values. Pseudocounts or non-integer count values can also be stored in the `counts` assay.

The rows of a `PhIPData` object represent peptides of interest and the columns represent samples. Sample and peptide metadata are stored in `DataFrame`s. Each row of the metadata `DataFrame` specifies the peptide/sample, and the columns represent different features associated with the peptides/samples. 

In addition to sample- and peptide-specific metadata, experimental metadata such as associated papers, experimental parameters, sequencing dates, etc. are stored in a list-like component named `metadata`. 

```{r, echo = FALSE, fig.cap = "Schematic of a PhIPData object. Commands used to access each component of the object are listed underneath its visual representation. Code in black indicates functions specific to `PhIPData` objects while functions in red extend `SummarizedExperiment` functions. Here, `pd` is a generic `PhIPData` object.", out.width = "\\maxwidth"}
knitr::include_graphics("extras/PhIPData.png")
```

# Creating a `PhIPData` object

```{r setup, warning = FALSE, message = FALSE}
library(dplyr)
library(readr)
```

To demonstrate the `PhIPData` class and functions, we will use a simulated example data set. Suppose we have PhIP-seq data from 5 individuals for 1288 peptides derived from known human viruses. 

```{r}
set.seed(20210120)

# Read in peptide metadata -------------
virscan_file <- system.file("extdata", "virscan.tsv", package = "PhIPData")
virscan_info <- readr::read_tsv(virscan_file,
                                col_types = readr::cols(
                                  pep_id = readr::col_character(),
                                  pro_id = readr::col_character(),
                                  pos_start = readr::col_double(),
                                  pos_end = readr::col_double(),
                                  UniProt_acc = readr::col_character(),
                                  pep_dna = readr::col_character(),
                                  pep_aa = readr::col_character(),
                                  pro_len = readr::col_double(),
                                  taxon_id = readr::col_double(),
                                  species = readr::col_character(),
                                  genus = readr::col_character(),
                                  product = readr::col_character()
                                )) %>%
  as.data.frame()

# Simulate data -------------
n_samples <- 5L
n_peps <- nrow(virscan_info)

counts_dat <- matrix(sample(1:1e6, n_samples*n_peps, replace = TRUE),
                 nrow = n_peps)
logfc_dat <- matrix(rnorm(n_samples*n_peps, mean = 0, sd = 10), 
                    nrow = n_peps)
prob_dat <- matrix(rbeta(n_samples*n_peps, shape1 = 1, shape2 = 1), 
                   nrow = n_peps)

# Sample metadata -------------
sample_meta <- data.frame(sample_name = paste0("sample", 1:n_samples),
                          gender = sample(c("M", "F"), n_samples, TRUE),
                          group = sample(c("ctrl", "trt", "beads"), n_samples, TRUE))

# Set row/column names -------------
rownames(counts_dat) <- rownames(logfc_dat) <-
  rownames(prob_dat) <- rownames(virscan_info) <-
  paste0("pep_", 1:n_peps)

colnames(counts_dat) <- colnames(logfc_dat) <-
  colnames(prob_dat) <- rownames(sample_meta) <-
  paste0("sample_", 1:n_samples)

# Experimental metadata -------------
exp_meta <- list(date_run = as.Date("2021/01/20"), 
                 reads_per_sample = colSums(counts_dat))
```

To create a `PhIPData` object, we will use the homonymous constructor `PhIPData()`.

```{r}
phip_obj <- PhIPData(counts_dat, logfc_dat, prob_dat, 
                     virscan_info, sample_meta, 
                     exp_meta)

phip_obj
```

The `PhIPData()` constructor is quite flexible; mismatched dimension names across assays and metadata are automatically corrected, and missing assays are initialized with empty matrices of the same dimensions. For more details on the constructor, type `help(PhIPData)`. 

# Accessing and modifying components of `PhIPData` object

## Assays

Assays store matrix-like data. For `PhIPData` objects, the assays `counts`, `logfc`, and `prob` are required. If any of these matrices were missing from the constructor, they are initialized with empty matrices of the same dimensions. Experimental data can be accessed via `assays(phip_obj)` or `assay(phip_obj, i)` command. `assays(phip_obj)` returns a list of all assays in the object, and list items can be accessed using the `$` or `[[` operators.

```{r}
assays(phip_obj)

head(assays(phip_obj)$counts)   # Returns the same as assays(phip_obj)[["counts"]]
```

While `assays(phip_obj)` returns a list of all assays in the `PhIPData` object, `assay(phip_obj, i)` returns a `matrix` of the specified assay. If `i` is missing, `assay(phip_obj)` defaults to the first assay (`counts`). `i` can be a character specifying the assay name or a numeric index of the assay.

```{r}
head(assay(phip_obj, "logfc"))   # Returns the same as assay(phip_obj, 2)
```

Since all `PhIPData` objects must contain the `counts`, `logfc`, and `prob` assays, we have defined three homonyous function to conveniently access and modify these assays.

```{r}
head(counts(phip_obj))
head(logfc(phip_obj))
head(prob(phip_obj))
```

After a `PhIPData` object has been created, data for new and existing assays can be set using `<-`. Dimension names of the replacement assays are automatically corrected to be identical to the object names. As we expect assays to contain homogenous data, replacement assays are coerced into matrices. Replacement assays must also be on the same dimension of the existing object.


```{r, error = T}
replacement_dat <- matrix(1, nrow = n_peps, ncol = n_samples)

# Replace the counts matrix -------------
head(counts(phip_obj))
counts(phip_obj) <- replacement_dat
head(counts(phip_obj))

# Add a new assay -----------
head(assay(phip_obj, "new_assay"))
assay(phip_obj, "new_assay") <- replacement_dat
head(assay(phip_obj, "new_assay"))

# Returns and error because `counts`, `logfc`, and `prob` must be in the 
# assays of a PhIPData object
assays(phip_obj) <- list(new_assay1 = replacement_dat, 
                         new_assay2 = replacement_dat)
```

## Peptide metadata

Information associated with peptides can be accessed using `peptideInfo(phip_obj)` or the inherited `rowRanges(phip_obj)` function. Both functions return a `GRanges` object. `GRanges` objects behave similar to matrices and can be subsetted using the usual 2-dimensional methods. More information about `GRanges` objects can be found [here](http://bioconductor.org/packages/release/bioc/html/GenomicRanges.html). 

Information about peptide positions in the protein sequence are stored as `IRanges` in the `GRanges` object. These are specified by `pos_start` and `pos_end` columns in the peptide metadata in the constructor. If these columns do not exist, the start and end positions are set to 0 by default. 

```{r}
# Only showing 2 columns for easier viewing
peptideInfo(phip_obj)[, 8:9]
```

## Sample metadata

Sample metadata describing the samples can be extracted from the `PhIPData` object using `sampleInfo(phip_obj)` or the inherited function `colData(phip_obj)`. Both functions return a `DataFrame` where each row corresponds to a sample and each column corresponds to some sample description. 

```{r}
sampleInfo(phip_obj)
```

Like in `SummarizedExperiments`/`RangedSummarizedExperiments`, sample metadata can be accessed using the `$` operator from the object. As demonstrated in the [Subsetting](#subsetting) section, this makes subetting data for a subgroup of samples very easy. 

```{r}
phip_obj$group
```

## Experimental metadata

Data associated with the experiment such as papers, date the samples were run, etc. can be accessed via the `metadata(phip_obj)` function. Experimental metadata is stored as a list, so it can be used to store *any* type of data. 

```{r}
metadata(phip_obj)
```

# Common operations on `PhIPData` objects

## Subsetting 

Like subsetting matrices and dataframes, `[` can be used for two-dimensional subsetting of `PhIPData` objects.

```{r}
phip_obj[1:10, 1:2]
```

As described in the [Sample metadata](#sample-metadata) section, `$` operates on the sample metadata column names, so we can also use `$` to select samples of a particular subgroups. 

```{r}
phip_obj[, phip_obj$group == "beads"]
```

In addition to subsetting by row indices, `PhIPData` supports subsetting rows using peptide metadata information. `subset(phip_obj, row_condition, column_condition)` returns a `PhIPData` object with rows where the specified condition holds. 

```{r}
ebv_sub <- subset(phip_obj, grepl("Epstein-Barr virus", species))

peptideInfo(ebv_sub)[, "species"]
```

To subset all beads-only samples from a `PhIPData` object, we can use the convenient wrapper function `subsetBeads()`:

```{r}
subsetBeads(phip_obj)
```

## `PhIPData` summaries

To assess the quality of the data, we are often interested in the number of reads per sample. This can be done using the `librarySize()` function. Names can be removed by setting the `withDimnames` parameter to `FALSE`. 

```{r}
librarySize(phip_obj)
```

The proportion of sample reads pulled by each peptide can also be obtained via `propReads()`. Like `librarySize()`, names can also be removed by setting `withDimnames` to `FALSE`. 

```{r}
head(propReads(phip_obj))
```

## Using template libraries

Rather than re-importing peptide annotations, `PhIPData` allows the user to create and reuse existing libraries. To save a library for future use, we can use `makeLibrary(object, name_of_library)`. The peptide metadata should be in a matrix-like form such as a `DataFrame` or `data.frame`. 

```{r}
# Save virscan_info as the human_virus library
makeLibrary(virscan_info, "human_virus")
```

The stored library can be accessed using `getLibrary(library_name)`. We can then use the stored library to construct a new `PhIPData` object as follows. 

```{r}
PhIPData(counts = counts_dat, 
         sampleInfo = sample_meta,
         peptideInfo = getLibrary("human_virus"))
```

Libraries can be removed with the `removeLibrary(library_name)` command.

```{r}
removeLibrary("human_virus")
```

## Using aliases

Peptides are often derived from species with long virus names. To quickly search up all viruses with "[Hh]uman immunodeficiency virus" in the species, you can create an alias of "HIV" to encode for the corresponding regex of interest. Alias's can be managed with:

* `getAlias(key)`: return the regex for the alias `key`. 
* `setAlias(key, pattern)`: create or modify the alias for `key`. If the key-pattern combination already exists in the database, then no changes are made. Otherwise, the pattern is replaced with `pattern`.
* `deleteAlias(key)`: remove an key-pattern combination from the alias database. 

```{r, error = T}
# Create alias for HIV ----------
setAlias("hiv", "[Hh]uman immunodeficiency virus")

# Use alias ----------
hiv_sub <- subset(phip_obj, grepl(getAlias("hiv"), species))
peptideInfo(hiv_sub)[, "species"]

# Remove alias from database -----------
deleteAlias("hiv")

# The following command returns an error that the virus does
# not exist in the alias database. 
subset(phip_obj, grepl(getAlias("hiv"), species))
```

Alias-pattern combinations are case senstive, so an entry with key "hiv" would differ from an entry with key "HIV." Like libraries, by default the location of the alias database is in the `extdata` folder of the `PhIPData` package. The location to an .rda file with the key-pattern values in a dataframe called alias can be retreived and specified using the `getAliasPath()` and `setAliasPath()` functions, respectively. 

## Coercion from `PhIPData` to other containers

As packages for identifying differential experession are also commonly used in analyzing PhIP-seq data, the `PhIPData` package supports coercion from `PhIPData` objects to `Lists`, `lists`, `DataFrame`, and `DGELists`. The function `as(phip_obj, "object_type")` converts a `PhIPData` object to a object of `object type`. 

For `DGELists`, the `group` slot is automatically populated with the `group` column in the sample metadata, if such a column exists. 

```{r}
# PhIPData to DGEList
as(phip_obj, "DGEList")
as(phip_obj, "DataFrame")
```

While `PhIPData` objects can be roughly constructed from `Lists`, `lists`, and `DGELists`, the coercion functions only construct a bare bones `PhIPData` object (with only `counts`, `logfc`, and `prob` assays and sample and peptide information). Any additional list/object components may be discarded. 

# `sessionInfo()`

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alias.R
\name{aliases}
\alias{aliases}
\alias{getAlias}
\alias{setAlias}
\alias{deleteAlias}
\title{Using aliases to subset virus data}
\usage{
getAlias(virus)

setAlias(virus, pattern)

deleteAlias(virus)
}
\arguments{
\item{virus}{character vector of the alias}

\item{pattern}{character vector of regexpressions corresponding to the alias}
}
\value{
\code{getAlias()} returns a vector of regexpressions corresponding to
queried inputs. The returned vector is the same length as the input vector.
Queries that do not exist in the database return \code{NA_character_}.
}
\description{
Rather than typing out full viruses names or repeating
regexpressions, users can use aliases as a convenient tool to subset
\code{PhIPData} objects by viral species.
}
\details{
Aliases are cached to an rda file containing only a
\code{data.frame} with two columns: \code{alias} and \code{pattern}. The
\code{alias} column contains the alias while the \code{pattern} column
contains the corresponding regexpression of interest.

Once an alias is added to the database, it can always be accessed once the
package is loaded. It is recommended to use the functions \code{setAlias}
and \code{deleteAlias}. If an alias already exists in the database,
\code{setAlias} replaces the matched pattern. If an alias does not exist
in the database, \code{getAlias} returns \code{NA_character_}.
}
\section{Functions}{
\itemize{
\item \code{getAlias}: return a regexpression corresponding to the alias.

\item \code{setAlias}: define/modify the regexpression for an alias.

\item \code{deleteAlias}: remove an alias from the database.
}}

\examples{

## Edit and modify aliases in the database
setAlias("test_virus", "test_pattern")
getAlias("test_virus")
setAlias("test_virus", "test_pattern2")
getAlias("test_virus")
deleteAlias("test_virus")

## Edit and modify multiple aliases at once.
setAlias(c("virus_1", "virus_2"), c("pattern_1", "pattern_2"))
getAlias(c("virus_1", "virus_2"))
deleteAlias(c("virus_1", "virus_2"))

## Example of how to subset HIV using `getAlias`
## Often, it is useful to set the `ignore.case` of `grep`/`grepl` to TRUE.
counts_dat <- matrix(1:10, nrow = 5)
peptide_meta <- data.frame(species = c(
    rep("Epstein-Barr virus", 2),
    rep("human immunodeficiency virus", 3)
))

phip_obj <- PhIPData(counts = counts_dat, peptideInfo = peptide_meta)
subset(phip_obj, grepl(getAlias("HIV"), species, ignore.case = TRUE))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset.R
\name{subsetBeads}
\alias{subsetBeads}
\title{Subset beads-only samples}
\usage{
subsetBeads(object)
}
\arguments{
\item{object}{\linkS4class{PhIPData} object}
}
\value{
a \linkS4class{PhIPData} object.
}
\description{
Function to subset PhIP-seq data for beads-only samples.
}
\examples{
example("PhIPData")
subsetBeads(phip_obj)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PhIPData-class.R
\name{PhIPData-methods}
\alias{PhIPData-methods}
\alias{counts,PhIPData-method}
\alias{logfc}
\alias{logfc,PhIPData-method}
\alias{prob}
\alias{prob,PhIPData-method}
\alias{peptideInfo}
\alias{peptideInfo,PhIPData-method}
\alias{sampleInfo}
\alias{sampleInfo,PhIPData-method}
\alias{assays<-,PhIPData,list-method}
\alias{assays<-,PhIPData,SimpleList-method}
\alias{assay<-,PhIPData,missing-method}
\alias{assay<-,PhIPData,numeric-method}
\alias{assay<-,PhIPData,character-method}
\alias{counts<-,PhIPData-method}
\alias{logfc<-}
\alias{logfc<-,PhIPData-method}
\alias{prob<-}
\alias{prob<-,PhIPData-method}
\alias{peptideInfo<-}
\alias{peptideInfo<-,PhIPData-method}
\alias{sampleInfo<-}
\alias{sampleInfo<-,PhIPData-method}
\title{Accessing and Modifying Information in PhIPData objects}
\usage{
\S4method{counts}{PhIPData}(object, ...)

logfc(object, ...)

\S4method{logfc}{PhIPData}(object, ...)

prob(object, ...)

\S4method{prob}{PhIPData}(object, ...)

peptideInfo(object, ...)

\S4method{peptideInfo}{PhIPData}(object, ...)

sampleInfo(object, ...)

\S4method{sampleInfo}{PhIPData}(object, ...)

\S4method{assays}{PhIPData,list}(x, withDimnames = TRUE, ...) <- value

\S4method{assays}{PhIPData,SimpleList}(x, withDimnames = TRUE, ...) <- value

\S4method{assay}{PhIPData,missing}(x, i, withDimnames = TRUE, ...) <- value

\S4method{assay}{PhIPData,numeric}(x, i, withDimnames = TRUE, ...) <- value

\S4method{assay}{PhIPData,character}(x, i, withDimnames = TRUE, ...) <- value

\S4method{counts}{PhIPData}(object, ...) <- value

logfc(object, ...) <- value

\S4method{logfc}{PhIPData}(object, ...) <- value

prob(object, ...) <- value

\S4method{prob}{PhIPData}(object, ...) <- value

peptideInfo(object) <- value

\S4method{peptideInfo}{PhIPData}(object) <- value

sampleInfo(object, ...) <- value

\S4method{sampleInfo}{PhIPData}(object) <- value
}
\arguments{
\item{object}{A \code{PhIPData} object}

\item{...}{parameters for \code{\link[SummarizedExperiment]{assays}},
which are typically not needed.}

\item{x}{A \code{PhIPData} object}

\item{withDimnames}{Parameter for
\linkS4class{RangedSummarizedExperiment} class functions. Overrided
since row/column names are automatically synced within each object.}

\item{value}{A \code{matrix}, \code{data.frame}, or \linkS4class{DataFrame}
of the same dimensions (not necessarily the same names)}

\item{i}{A \code{numeric}, {character}}
}
\value{
Accessors: a \link{DataFrame} object

Setters: a \code{PhIPData} object
}
\description{
Methods to extract and modify \code{assay(s)}(including
convenient functions for \code{counts}, \code{logfc}, and \code{prob}),
\code{sampleInfo}, \code{peptideInfo}, and \code{metadata}.
}
\details{
In addition to the functions detailed in
\linkS4class{RangedSummarizedExperiment}, the \code{PhIPData} class includes
conveniently named functions to quickly access and modify frequently used
components of PhIPData objects.

Replacement functions ensure that names of the replacement object are matched
with the names of the \code{PhIPData} object.

Since packages for identifying differential expression in RNA-seq experiments
are frequently used for estimating fold-changes for peptide enrichments,
the class also includes coercion methods to and from \linkS4class{DGEList}s.
}
\section{Available methods}{

In the following code snippets, \code{x} is a \linkS4class{PhIPData} object,
\code{value} is a matrix-like object with the same dimensions as \code{x},
and \code{...} are further arguments passed to \code{\link{assay}}
(for the getter) or \code{\link{assay<-}} (for the setter).
\describe{
\item{\code{counts(x, ...)}, \code{counts(x, ...) <- value}:}{
Get or set a matrix of raw read counts
}
\item{\code{logfc(x, ...)}, \code{logfc(x, ...) <- value}:}{
Get or set a matrix of log2 estimated fold changes (in comparison to
beads-only samples)
}
\item{\code{prob(x, ...)}, \code{pob(x, ...) <- value}:}{
Get or set a matrix of probabilities associated with whether
         a sample has an enriched antibody response for a peptide.
}
}
}

\examples{
example("PhIPData")

replacement_dat <- matrix(1L, nrow = 5, ncol = 5)

## SummarizedExperiment Accessors and Setters
assays(phip_obj)
assays(phip_obj)$counts <- replacement_dat
assay(phip_obj, "logfc")
assay(phip_obj, "logfc") <- replacement_dat

## counts
counts(phip_obj)
counts(phip_obj) <- counts_dat

## logfc
logfc(phip_obj)
logfc(phip_obj) <- logfc_dat

## prob
prob(phip_obj)
prob(phip_obj) <- replacement_dat

## coercion functions
as(phip_obj, "DGEList")
as(phip_obj, "List")
as(phip_obj, "list")
}
\seealso{
\code{\link[SummarizedExperiment]{assays}} for
\linkS4class{SummarizedExperiment} operations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defineBeads.R
\name{defineBeads}
\alias{defineBeads}
\alias{getBeadsName}
\alias{setBeadsName}
\title{Defining how beads-only samples are encoded.}
\usage{
getBeadsName()

setBeadsName(name)
}
\arguments{
\item{name}{a string indicating how beads-only samples are encoded.}
}
\value{
a string indicating how beads-only samples are encoded.
}
\description{
\code{getBeadsName} and \code{setBeadsName} are two function to
get and set the string that encodes which samples are beads-only samples.
Information about beads-only samples are stored in the \code{groups} column
of \code{sampleInfo}.
}
\details{
If \code{name} is of length greater than one, only the first element
of the vector is used. Non-character values of \code{name} are first coerced
into strings.
}
\section{Functions}{
\itemize{
\item \code{getBeadsName}: function that returns a string corresponding to how
beads-only samples are encoded.

\item \code{setBeadsName}: function to set the string that indicates which
samples are beads-only samples in the \code{groups} column of
\code{sampleInfo}.
}}

\examples{
## Returns the default string, "beads"
getBeadsName()

## Not run since it changes defaults/user settings
\dontrun{
setBeadsName("beads-only")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaries.R
\name{librarySize}
\alias{librarySize}
\title{Calculate total read counts for each sample.}
\usage{
librarySize(object, ..., withDimnames = TRUE)
}
\arguments{
\item{object}{\linkS4class{PhIPData} object}

\item{...}{arguments passed to \link{colSums}}

\item{withDimnames}{logical; if true, the vector names are the sample names;
otherwise the vector is unnamed.}
}
\value{
a (named) numeric vector. The length of the vector is equal to the
number of samples.
}
\description{
This function is a wrapper function for \link{colSums} on the \code{counts}
assay.
}
\examples{
example("PhIPData")
librarySize(phip_obj)

## Return an unnamed vector
librarySize(phip_obj, withDimnames = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaries.R
\name{propReads}
\alias{propReads}
\title{Proportion of sample reads}
\usage{
propReads(object, withDimnames = TRUE)
}
\arguments{
\item{object}{\linkS4class{PhIPData} object}

\item{withDimnames}{logical; if true return a matrix with the same dimension
names as the original object.}
}
\value{
A (named) numeric matrix with the same dimensions as the function
input. Matrix values are between 0 and 1.
}
\description{
This function calculates the proportion of total sample reads pulled by
each peptide.
}
\examples{
example("PhIPData")
propReads(phip_obj)

## Return an unnamed matrix
propReads(phip_obj, withDimnames = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PhIPData-class.R
\docType{class}
\name{PhIPData-class}
\alias{PhIPData-class}
\alias{.PhIPData}
\alias{PhIPData}
\title{The PhIPData class}
\usage{
PhIPData(
  counts = matrix(nrow = 0, ncol = 0),
  logfc = matrix(nrow = 0, ncol = 0),
  prob = matrix(nrow = 0, ncol = 0),
  peptideInfo = S4Vectors::DataFrame(),
  sampleInfo = S4Vectors::DataFrame(),
  metadata = list(),
  .defaultNames = "info"
)
}
\arguments{
\item{counts}{a \code{matrix}, \code{data.frame}, or
\code{\linkS4class{DataFrame}}of \strong{integer} read counts.}

\item{logfc}{a \code{matrix}, \code{data.frame}, or
\code{\linkS4class{DataFrame}} of log2 estimated fold changes.}

\item{prob}{a \code{matrix}, \code{data.frame}, or
\code{\linkS4class{DataFrame}} of probability values (p-values or
posterior probabilities) for enrichment estimates.}

\item{peptideInfo}{a \code{data.frame} or \code{\linkS4class{DataFrame}} of
peptide information.}

\item{sampleInfo}{a \code{data.frame} or \code{\linkS4class{DataFrame}} of
additional sample information.}

\item{metadata}{a \code{list} object containing experiment-specific metadata.}

\item{.defaultNames}{vector of names to use when sample and peptide
    identifiers disagree across the metadata and the \code{counts},
    \code{logfc}, and \code{prob} matrices. If \code{.defaultNames} is of
    length 1, the same source is used for both peptide and sample
    identifiers. If \code{.defaultNames} is longer than 2, the first and
    second elements correspond to the names for peptides and samples,
    respectively.

    Valid options are:
    \itemize{
        \item{"info": }{names should be taken from the \code{SampleInfo} or
            \code{peptideInfo} objects.}
        \item{"counts": }{names should be taken from the row/column names
            of the \code{counts} object.}
        \item{"logfc": }{names should be taken from the row/column names of
            the \code{logfc} object.}
        \item{"prob": }{names should be taken from the row/column names of
            the \code{prob} object.}
    }}
}
\value{
A \code{PhIPData} object.
}
\description{
The \code{PhIPData} class is a matrix-like container designed to organize
results from phage-immunoprecipitation (PhIP-Seq) experiments. Rows in
PhIPData objects represent peptides and columns represent samples. Each
object contains at least three assays:

\itemize{
    \item{\code{counts}:} a matrix of raw read counts,
    \item{\code{logfc}:} a matrix of log2 estimated fold-change in
         comparison to beads-only samples,
    \item{\code{prob}:} a matrix of probabilities associated with whether
         a sample has an enriched antibody response for a peptide.
}

The \code{PhIPData} class extends the
\linkS4class{RangedSummarizedExperiment} class, so methods documented
in \linkS4class{RangedSummarizedExperiment} and
\linkS4class{SummarizedExperiment} also work on \code{PhIPData} objects.
}
\details{
Rows of \code{PhIPData} objects correspond to peptides of interest and are
organized in \link{GRanges} or \link{GRangesList} objects. Though originally
designed for genomic ranges, the sequence name and genomic range information
in \link{GRanges} objects can be replaced with peptide names and amino acid
positions, respectively. If no peptide names are given, peptides are given
the names of \code{pep_rownum}. Peptide positions are specified by columns
\code{pos_start} and \code{pos_end} in the \code{peptideInfo} argument of the
constuctor. Missing position information is set to 0. Additional peptide
annotation can also be stored in \link{GRanges} objects and can be used
to subset \code{PhIPData} objects as shown below.

Columns of \code{PhIPData} objects represent samples. Sample metadata
are stored in a \link{DataFrame} and can be accessed as shown below. If no
sample names are specified, samples are given default names of
\code{sample_colnum}.

Unlike \link{RangedSummarizedExperiment}/\link{SummarizedExperiment} objects,
\code{PhIPData} objects must contain \code{counts}, \code{logfc},
\code{prob}. If any of the three assays are missing when the constructor is
called, an empty matrix of the same names and dimensions is initialized for
that assay. Sample and peptide names are harmonized across assays and
annotation during construction and replacement.

Though `counts` typically contain integer values for the number of reads
aligned to each peptide, `PhIPData` only requires that stored values are
non-negative numeric values. Pseudocounts or non-integer count values can
also be stored in the `counts` assay.
}
\section{Constructor}{

\code{PhIPData} objects are constructed using the homonymous function and
arguments as described above. Any \code{PhIPData} object can be created
so long as peptide and sample identifiers (or lack thereof) are specified
via any of the parameters.
}

\examples{
## Construct a new PhIPData object
counts_dat <- matrix(sample(1:1e6, 25, replace = TRUE), nrow = 5)
logfc_dat <- matrix(rnorm(25, 0, 10), nrow = 5)
prob_dat <- matrix(rbeta(25, 1, 1), nrow = 5)

peptide_meta <- data.frame(
    pos_start = 1:5,
    pos_end = 6:10,
    species = c(rep("HIV", 3), rep("EBV", 2))
)
sample_meta <- data.frame(
    gender = sample(c("M", "F"), 5, TRUE),
    group = sample(c("ctrl", "trt", "beads"), 5, TRUE)
)
exp_meta <- list(
    date_run = as.Date("2021/01/20"),
    reads_per_sample = colSums(counts_dat)
)

rownames(counts_dat) <- rownames(logfc_dat) <-
    rownames(prob_dat) <- rownames(peptide_meta) <-
    paste0("pep_", 1:5)
colnames(counts_dat) <- colnames(logfc_dat) <-
    colnames(prob_dat) <- rownames(sample_meta) <-
    paste0("sample_", 1:5)

phip_obj <- PhIPData(
    counts_dat, logfc_dat, prob_dat,
    peptide_meta, sample_meta, exp_meta
)
phip_obj
}
\seealso{
\code{\link{PhIPData-methods}} for accessors and modifiers for PhIPData
     components.
     \linkS4class{SummarizedExperiment}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/library.R
\name{peptideLibraries}
\alias{peptideLibraries}
\alias{getLibrary}
\alias{makeLibrary}
\alias{removeLibrary}
\alias{listLibrary}
\title{Peptide libraries}
\usage{
getLibrary(name)

makeLibrary(library, name)

removeLibrary(name)

listLibrary()
}
\arguments{
\item{name}{name of the library}

\item{library}{a \code{matrix}, \code{data.frame}, or
\linkS4class{DataFrame} with the peptide information for the
specified library.}
}
\value{
\code{getLibrary} returns a \linkS4class{DataFrame} corresponding to
the peptide information for the specified library.
}
\description{
PhIP-Seq experiments often use identical peptide libraries
different cohorts. These functions enable the user to conveniently reuse
tidied libraries.
}
\details{
Each library is stored as a \linkS4class{DataFrame} in .rds file.
New libraries can be stored for future use with the \code{makeLibrary}
function.
}
\section{Functions}{
\itemize{
\item \code{getLibrary}: return a \linkS4class{DataFrame} with the
peptide information corresponding to the library.

\item \code{makeLibrary}: create and store a \linkS4class{DataFrame} with
the specified peptide information.

\item \code{removeLibrary}: delete stored libraries

\item \code{listLibrary}: list all available libraries
}}

\examples{
## Create a new library
pep_meta <- data.frame(species = c(
    rep("human immunodeficiency virus", 3),
    rep("Epstein-Barr virus", 2)
))
makeLibrary(pep_meta, "new_library")

## Use new library
counts_dat <- matrix(1:10, nrow = 5)
phip_obj <- PhIPData(
    counts = counts_dat,
    peptideInfo = getLibrary("new_library")
)

## List libraries
listLibrary()

## Delete created library
removeLibrary("new_library")

}
